/*******************************************************************************
  File : evaluate.c

  Description : Module d'valuation statique d'une position.
*******************************************************************************/

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte des diffrents modules du programme : */
#include "debug.h"
#include "evaluate.h"
#include "uci.h"

/*******************************************************************************
  Constants :
*******************************************************************************/

/* Bonus et malus pour les pions : */
const int bonus_connected_pawn[3] = { 4, 8, 12};
const int bonus_passed_pawn[3] = { 2, 4, 6 };
const int bonus_strong_square = 4;
const int malus_backward_pawn[3] = { -2, -4, -6 };
const int malus_isolated_pawn[3] = { -2, -4, -6 };
const int malus_weak_backward_pawn[3] = { -4, -4, -4 };
const int malus_weak_isolated_pawn[3] = { -8, -8, -8 };
const int malus_doubled_pawn[3] = { -1, -2, -4 };
const int malus_blocked_pawns[2] = { -8, -4 };

/* Bonus pour les fous : */
const int bonus_bishop_pair = 4;

/* Malus pour la dame : */
const int malus_queen_opening = -4;

/* Bonus pour les tours : */
const int bonus_rook_half_open_file = 4;
const int bonus_rook_open_file = 4;
const int bonus_rook_seventh_rank[3] = { 4, 8, 12};
const int bonus_heavy_pieces_doubling = 2;

/* Bonus et malus pour le roi : */
const int bonus_castling[2] = { 16, 24 };
const int malus_king_cover[2] = { -4, -8 };
const int malus_king_no_cover[2] = { -8, -16 };
const int malus_no_castling[2] = { -4, -8 };

/* Bonus et malus pour encourager, ou pas, les changes en finale : */
const int bonus_exchange = 16;
const int malus_exchange = -8;

/* Malus pour viter les rptitions lorsque le programme a l'avantage : */
const int malus_repetition = -16;

/* Bonus pour la position du pion (favorise l'avancement du pion) : */
const int bonus_pawn[9][9] = {
  { 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 1, 2, 3, 4, 5, 0 },
  { 0, 0, 0, 2, 4, 6, 8,10, 0 },
  { 0, 0, 0, 3, 6, 9,12,15, 0 },
  { 0, 0, 0, 4, 8,12,16,20, 0 },
  { 0, 0, 0, 4, 8,12,16,20, 0 },
  { 0, 0, 0, 3, 6, 9,12,15, 0 },
  { 0, 0, 0, 2, 4, 6, 8,10, 0 },
  { 0, 0, 0, 1, 2, 3, 4, 5, 0 }
};

/* Bonus pour la position du cavalier : */
const int bonus_knight[9][9] = {
  { 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0,-4, 0, 0, 0, 0, 0, 0,-4 },
  { 0, 0, 0, 4, 4, 4, 4, 0, 0 },
  { 0, 0, 4, 8, 8, 8, 8, 4, 0 },
  { 0, 0, 4, 8, 8, 8, 8, 4, 0 },
  { 0, 0, 4, 8, 8, 8, 8, 4, 0 },
  { 0, 0, 4, 8, 8, 8, 8, 4, 0 },
  { 0, 0, 0, 4, 4, 4, 4, 0, 0 },
  { 0,-4, 0, 0, 0, 0, 0, 0,-4 }
};

/* Bonus pour la position du fou : */
const int bonus_bishop[9][9] = {
  { 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0,-4, 0, 0, 0, 0, 0, 0,-4 },
  { 0, 0, 4, 4, 4, 4, 4, 4, 0 },
  { 0, 0, 4, 8, 8, 8, 8, 4, 0 },
  { 0, 0, 4, 8, 4, 4, 8, 4, 0 },
  { 0, 0, 4, 8, 4, 4, 8, 4, 0 },
  { 0, 0, 4, 8, 8, 8, 8, 4, 0 },
  { 0, 0, 4, 4, 4, 4, 4, 4, 0 },
  { 0,-4, 0, 0, 0, 0, 0, 0,-4 }
};

/* Bonus pour la position du roi (dbut et milieu de partie) : */
const int bonus_king[9][9] = {
  { 0,  0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  4,  0, -4, -8,-12,-16,-20,-24 },
  { 0, 12,  0, -4, -8,-12,-16,-20,-24 },
  { 0,  8,  0, -4, -8,-12,-16,-20,-24 },
  { 0,  0,  0, -4, -8,-12,-16,-20,-24 },
  { 0,  0,  0, -4, -8,-12,-16,-20,-24 },
  { 0,  8,  0, -4, -8,-12,-16,-20,-24 },
  { 0, 12,  0, -4, -8,-12,-16,-20,-24 },
  { 0,  4,  0, -4, -8,-12,-16,-20,-24 }
};

/* Bonus pour la centralisation du roi en finale : */
const int bonus_king_endgame[9][9] = {
  { 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0,-8,-4, 0, 0, 0, 0,-4,-8 },
  { 0,-4, 0, 0, 0, 0, 0, 0,-4 },
  { 0, 0, 0, 4, 4, 4, 4, 0, 0 },
  { 0, 0, 0, 4, 8, 8, 4, 0, 0 },
  { 0, 0, 0, 4, 8, 8, 4, 0, 0 },
  { 0, 0, 0, 4, 4, 4, 4, 0, 0 },
  { 0,-4, 0, 0, 0, 0, 0, 0,-4 },
  { 0,-8,-4, 0, 0, 0, 0,-4,-8 }
};

/*******************************************************************************
  Data :
*******************************************************************************/

/* Pour mmoriser le nombre de fous de chaque couleur : */
int bishops_count[2];

/* Pour marquer les cases fortes : */
int bonus_square[9][9];

/*******************************************************************************
  External functions :
*******************************************************************************/

/*******************************************************************************
  Function     : check_game_phase
  Description  : Evaluation de la phase de la partie pour une couleur donne.
  Parameters   : aucun
  Return value : phase de la partie
  Validation   : 1
*******************************************************************************/
int check_game_phase (void) {

/* ------------------------------- Local data ------------------------------- */

int color;                   /* Couleur du camp. */
int game_phase = OPENING;
int pieces_move;
int x;                       /* Colonne de l'chiquier. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("check_game_phase", false, &check_game_phase_calls);
#endif /* DEBUG */

  /* Si un grand nombre de pices ont t changes, */
  if (piece_count < 5) {

    /* alors la partie est dans la phase finale. */
    game_phase = ENDGAME;
  }
  else {

    /* Cas de passage en milieu de partie pour la couleur au trait :
       1) Le roque est effectu (le cas le plus frquent).
       2) Le roque n'est plus possible et au moins 3 pices (mineures + dame)
          ont t dplaces.
       3) Au moins 30 demi-coups ont t jous. */
    for (color = BLACK; color <= WHITE && game_phase == OPENING; color ++) {

      pieces_move = 0;

      /* Calcule le nombre de pices qui ont boug pour la couleur au trait : */
      for (x = B; x <= G; x++) {

        if (moved[(RR(1) + 1) * 10 + x]) {

          pieces_move++;
        }
      }

      /* Si l'un des cas prcit est rencontr, */
      if (   (CASTLE_STATUS & castle_done_mask[color])
          || (   !(CASTLE_STATUS & castle_by_color_mask[color])
              && pieces_move >= 3)
          || (game_ply > 20)) {

        /* alors c'est la phase du milieu de partie. */
        game_phase = MIDDLE;
      }
    }
  }

  return game_phase;
}
/* End of function : check_game_phase */

/*******************************************************************************
  Function     : evaluate_position
  Description  : Evaluation statique d'une position.
  Parameters   : aucun
  Return value : valuation de la position
  Validation   : 1
*******************************************************************************/
int evaluate_position (void) {

/* ------------------------------- Local data ------------------------------- */

bool backward;          /* Indicateur si un pion est arrir. */
bool isolated;          /* Indicateur si un pion est isol. */

int cf;                 /* color_factor : Pour prendre en compte la couleur. */
int color;              /* Couleur du camp. */
int gs;                 /* game_phase : Phase de jeu (ouverture, milieu ou
                           finale) */
int heavy_pieces[2][9]; /* Pour tester le doublement des pices lourdes */
int i, j;               /* Indices de boucle. */
int pawn_rank[2][10];   /* Pour mmoriser la range des pions arrirs */
int pawns[2][10];       /* Pour le comptage des pions par colonne et couleur. */
int score = 0;          /* Evaluation de la position. */
int x, y;               /* Colonne et range de l'chiquier. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("evaluate_position", true, &evaluate_position_calls);
#endif /* DEBUG */

  /* Initialisation des tables d'valuation : */
  memset (bishops_count, 0, sizeof (bishops_count));
  memset (bonus_square, 0, sizeof (bonus_square));
  memset (heavy_pieces, 0, sizeof (heavy_pieces));
  memset (pawns, 0, sizeof (pawns));

  /* Initialisation des tables de mmorisation de la range des pions. */
  for (i = 0; i < 10; i++) {

    pawn_rank[WHITE][i] = 7;
    pawn_rank[BLACK][i] = 2;
  }

  /* Pour les pices noires puis blanches, une premire passe est faite, */
  for (color = BLACK; color <= WHITE; color++) {

    /*  partir de la table des pices, */
    for (j = 1; j <= pieces_number[color]; j++) {

      /* si une pice est prsente, */
      if (pieces[color][j]) {

        x = file[pieces[color][j]];
        y = rank[pieces[color][j]];

        /* on vrifie si elle sert  dterminer un critre positionnel. */
        switch (board[pieces[color][j]]) {

          case (BPAWN) :
          case (WPAWN) :

            /* Mmorise la prsence d'un pion sur la colonne. */
            pawns[color][x]++;

            /* Mmorise la range du pion le moins avanc sur la colonne. */
            if (RR(y) < RR(pawn_rank[color][x])) {

              pawn_rank[color][x] = y;
            }
            break;

          case (BBISHOP) :
          case (WBISHOP) :

            /* Mmorise la prsence de la paire de fous. */
            bishops_count[color]++;
            break;

          case (BROOK) :
          case (WROOK) :
          case (BQUEEN) :
          case (WQUEEN) :

            /* Mmorise les pices lourdes de mme couleur par colonne. */
            heavy_pieces[color][x]++;
            break;
        }
      }
    }
  }

#ifdef DEBUG
DB_EVAL_1
#endif /* DEBUG */

  /* Identifie la phase de la partie (ouverture, milieu ou finale). */
  gs = evaluate.check_game_phase ();

  /* Analyse de la position des pices blanches puis noires. */
  for (color = WHITE; color >= BLACK; color--) {

    /* Permet de tenir compte qu'un avantage pour un camp est en fait un
       dsavantage pour le camp adverse, aussi un bonus doit tre pris comme un
       malus et vice versa. */
    cf = color ? 1 : -1;

    /* Pour toutes les pices connues dans la table des pices, */
    for (j = 1; j <= pieces_number[color]; j++) {

      /* si elle est encore prsente sur l'chiquier, */
      if (pieces[color][j]) {

        x = file[pieces[color][j]];
        y = rank[pieces[color][j]];

        /* alors le programme calcule son influence sur la position. */
        switch (board[pieces[color][j]]) {

          case (BPAWN) :
          case (WPAWN) :

            /* Par dfaut un pion n'est ni arrir, ni isol. */
            backward = false;
            isolated = false;

            /* Bonus pour la valeur du pion. */
            score += pieces_value[BPAWN] * cf;
#ifdef DEBUG
DB_EVAL("pawn", "value")
#endif /* DEBUG */

            /* Bonus pour favoriser l'avance vers la case de promotion. */
            score += bonus_pawn[x][RR(y)] * cf;
#ifdef DEBUG
DB_EVAL("pawn", "position")
#endif /* DEBUG */
            /* Si les pions adjacents sont plus avancs, */
            if (   RR(pawn_rank[color][x + 1]) > RR(y)
                && RR(pawn_rank[color][x - 1]) > RR(y)) {

              /* alors le pion est arrir. */
              backward = true;

              /* Si l'ouverture est termine ou si le pion n'est plus sur sa
                 range initiale, */
              if (gs != OPENING || RR(y) != 2) {

                /* alors un malus est ajout  la position. */
                score += malus_backward_pawn[gs] * cf;
#ifdef DEBUG
DB_EVAL("pawn", "backward")
#endif /* DEBUG */
              }

              /* La case situe devant le pion est une case forte pour
                 une pice adverse. */
              bonus_square[x][y + 1 * cf] += bonus_strong_square * cf;

              /* Si les pions adjacents ont disparu, */
              if (   !pawns[color][x + 1]
                  && !pawns[color][x - 1]) {

                /* alors le pion est isol, */
                isolated = true;

                /* et un malus supplmentaire est appliqu. */
                score += malus_isolated_pawn[gs] * cf;
#ifdef DEBUG
DB_EVAL("pawn", "isolated")
#endif /* DEBUG */
              }
            }

            /* Si le pion est sur une colonne semi-ouverte, */
            if (!pawns[color^1][x]) {

              /* s'il est arrir : */
              if (backward) {

                /* alors un malus est appliqu. */
                score += malus_weak_backward_pawn[gs] * cf;
#ifdef DEBUG
DB_EVAL("pawn", "weak backward")
#endif /* DEBUG */
              }

              /* S'il est isol, */
              if (isolated) {

                /* alors un malus plus important est appliqu. */
                score += malus_weak_isolated_pawn[gs] * cf;
#ifdef DEBUG
DB_EVAL("pawn", "weak isolated")
#endif /* DEBUG */
              }
            }

            /* Les pions doubls, voire tripls, */
            if (pawns[color][x] > 1) {

              /* sont lourdement pnaliss. */
              score += malus_doubled_pawn[gs] * cf * (pawns[color][x] - 1);
#ifdef DEBUG
DB_EVAL("pawn", "doubled")
#endif /* DEBUG */
            }

            /* Un pion pass (aucun pion adverse en face et sur les colonnes
               adjacentes), */
            if (   !pawns[color^1][x]
                && RR(y) >= RR(pawn_rank[color^1][x - 1])
                && RR(y) >= RR(pawn_rank[color^1][x + 1])) {

              /* est favoris, et ce d'autant plus qu'il est avanc. */
              score += bonus_passed_pawn[gs] * cf * bonus_pawn[x][RR(y)];
#ifdef DEBUG
DB_EVAL("pawn", "passed")
#endif /* DEBUG */
              /* S'il est soutenu par un pion adjacent, */
              if (   !isolated
                  && (   (RR(pawn_rank[color][x]) -
                          RR(pawn_rank [color][x + 1]) == 1)
                      || (RR(pawn_rank[color][x]) -
                          RR(pawn_rank [color][x - 1]) == 1))) {

                /* il obtient un bonus supplmentaire. */
                score += bonus_connected_pawn[gs] * cf;
#ifdef DEBUG
DB_EVAL("pawn", "connected")
#endif /* DEBUG */
              }
            }
            break;

          case (BKNIGHT) :
          case (WKNIGHT) :

            /* Bonus pour la valeur du cavalier. */
            score += pieces_value[BKNIGHT] * cf;
#ifdef DEBUG
DB_EVAL("knight", "value")
#endif /* DEBUG */
            /* Bonus pour la position du cavalier. */
            score += bonus_knight[x][y] * cf;
#ifdef DEBUG
DB_EVAL("knight", "position")
#endif /* DEBUG */
            /* Bonus si le cavalier occupe une case forte. */
            score += bonus_square[x][y];
#ifdef DEBUG
DB_EVAL("knight", "strong square")
#endif /* DEBUG */
            break;

          case (BBISHOP) :
          case (WBISHOP) :

            /* Bonus pour la valeur du fou. */
            score += pieces_value[BBISHOP] * cf;
#ifdef DEBUG
DB_EVAL("bishop", "value")
#endif /* DEBUG */
            /* Bonus pour la position du fou. */
            score += bonus_bishop[x][y] * cf;
#ifdef DEBUG
DB_EVAL("bishop", "position")
#endif /* DEBUG */
            /* Bonus si le fou occupe une case forte. */
            score += bonus_square[x][y];
#ifdef DEBUG
DB_EVAL("bishop", "strong square")
#endif /* DEBUG */
            /* A partir du milieur de partie, */
            if (   gs != OPENING
               && bishops_count[color] == 2) {

              /* la paire de fous obtient un bonus. */
              score += bonus_bishop_pair * cf;
#ifdef DEBUG
DB_EVAL("bishop", "pair")
#endif /* DEBUG */
            }
            break;

          case (BROOK) :
          case (WROOK) :

            /* Bonus pour la valeur de la tour. */
            score += pieces_value[BROOK] * cf;
#ifdef DEBUG
DB_EVAL("rook", "value")
#endif /* DEBUG */
            /* La septime range est intressante pour une tour. */
            if (RR(y) == 7) {

              score += bonus_rook_seventh_rank[gs] * cf;
#ifdef DEBUG
DB_EVAL("rook", "seventh rank")
#endif /* DEBUG */
            }

            /* Accorde un bonus pour une tour, */
            if (!pawns[color][x]) {

              /* sur une colonne semi-ouverte :  */
              score += bonus_rook_half_open_file * cf;
#ifdef DEBUG
DB_EVAL("rook", "half open file")
#endif /* DEBUG */
              if (!pawns[color^1][x]) {

                /* et mieux sur une colonne ouverte :  */
                score += bonus_rook_open_file * cf;
#ifdef DEBUG
DB_EVAL("rook", "open file")
#endif /* DEBUG */
              }
            }

            /* Le doublement des pices lourdes sur une mme colonne, */
            if (heavy_pieces[color][x] > 1) {

              /* obtient un bonus. */
              score += bonus_heavy_pieces_doubling * cf;
#ifdef DEBUG
DB_EVAL("rook", "doubling")
#endif /* DEBUG */
            }
            break;

          case (BQUEEN) :
          case (WQUEEN) :

            /* Bonus pour la valeur de la pice. */
            score += pieces_value[BQUEEN] * cf;
#ifdef DEBUG
DB_EVAL("queen", "value")
#endif /* DEBUG */
            /* Dans l'ouverture, */
            if (gs == OPENING) {

              /* le dplacement de la dame avant les pices mineures, */
              if (   moved[94 - 70 * color]) {

                if (   !moved[92 - 70 * color]
                    || !moved[93 - 70 * color]
                    || !moved[96 - 70 * color]
                    || !moved[97 - 70 * color]) {

                  /* est pnalis. */
                  score += malus_queen_opening * cf;
#ifdef DEBUG
DB_EVAL("queen", "move in opening")
#endif /* DEBUG */
                }
              }
            }

            /* Le doublement des pices lourdes sur une mme colonne, */
            if (heavy_pieces[color][x] > 1) {

              /* obtient un bonus. */
              score += bonus_heavy_pieces_doubling * cf;
#ifdef DEBUG
DB_EVAL("queen", "doubling")
#endif /* DEBUG */
            }
            break;

          case (BKING) :
          case (WKING) :

            /* En finale, */
            if (gs == ENDGAME) {

              /* le roi peut tre jou comme une autre pice. */
              score += bonus_king_endgame[x][y] * cf;
#ifdef DEBUG
DB_EVAL("king", "endgame value")
#endif /* DEBUG */
            }
            else {

              /* sinon il doit tre protg. */
              score += bonus_king[x][RR(y)] * cf;
#ifdef DEBUG
DB_EVAL("king", "position")
#endif /* DEBUG */
              /* Si le roi a roqu, */
              if (CASTLE_STATUS & castle_done_mask[color]) {

                /* alors il obtient un bonus. */
                score += bonus_castling[gs] * cf;
#ifdef DEBUG
DB_EVAL("king", "castling")
#endif /* DEBUG */
              }
              /* par contre s'il ne peut plus roquer, */
              else if (!(CASTLE_STATUS & castle_by_color_mask[color])) {

                /* il obtient un malus. */
                score += malus_no_castling[gs] * cf;
#ifdef DEBUG
DB_EVAL("king", "no castling")
#endif /* DEBUG */
              }

              /* Examen de la protection du roi offerte par les pions situs
                 devant lui. */

              /* Si le pion situ sur la mme colonne est devant le roi, */
              if (pawns[color][x] && RR(y) < RR(pawn_rank[color][x])) {

                /* alors il obtient un malus s'il est trop loign,
                   surtout dans l'ouverture. */
                score +=   malus_king_cover[gs] * cf
                         * (RR(pawn_rank[color][x]) - RR(y) - 1);
#ifdef DEBUG
DB_EVAL("king", "outpost front pawn")
#endif /* DEBUG */
              }
              else {

                /* sinon il y a un malus pour absence de couverture. */
                score += malus_king_no_cover[gs] * cf;
#ifdef DEBUG
DB_EVAL("king", "no cover from front pawn")
#endif /* DEBUG */
              }

              /* Si le pion situ  droite est devant le roi, */
              if (pawns[color][x + 1] && RR(y) < RR(pawn_rank[color][x + 1])) {

                /* alors il obtient un malus s'il est trop loign,
                   surtout dans l'ouverture. */
                score +=   malus_king_cover[gs] * cf
                         * (RR(pawn_rank[color][x + 1]) - RR(y) - 1);
#ifdef DEBUG
DB_EVAL("king", "outpost right pawn")
#endif /* DEBUG */
              }
              else {

                /* sinon il y a un malus pour absence de couverture. */
                score += malus_king_no_cover[gs] * cf;
#ifdef DEBUG
DB_EVAL("king", "no cover from right pawn")
#endif /* DEBUG */
              }

              /* Si le pion situ  gauche est devant le roi, */
              if (pawns[color][x - 1] && RR(y) < RR(pawn_rank[color][x - 1])) {

                /* alors il obtient un malus s'il est trop loign,
                   surtout dans l'ouverture. */
                score +=   malus_king_cover[gs] * cf
                         * (RR(pawn_rank[color][x - 1]) - RR(y) - 1);
#ifdef DEBUG
DB_EVAL("king", "outpost left pawn")
#endif /* DEBUG */
              }
              else {

                /* sinon il y a un malus pour absence de couverture. */
                score += malus_king_no_cover[gs] * cf;
#ifdef DEBUG
DB_EVAL("king", "no cover from left pawn")
#endif /* DEBUG */
              }
            }
            break;
        }
      }
    }

    /* En dbut de partie, */
    if (gs == OPENING || gs == MIDDLE) {

      /* le programme pnalise le blocage des pions centraux par une pice amie
         situe sur la case devant lui. */

      /* Pion d bloqu.  */
      if (!moved[84 - 50 * color] && board[74 - 30 * color]
          && ((board[74 - 30 * color] >> 3) == color)) {

        score += malus_blocked_pawns[gs] * cf;
#ifdef DEBUG
DB_EVAL("pawn", "d pawn is blocked")
#endif /* DEBUG */
      }

      /* Pion e bloqu. */
      if (!moved[85 - 50 * color] && board[75 - 30 * color]
          && ((board[75 - 30 * color] >> 3) == color)) {

        score += malus_blocked_pawns[gs] * cf;
#ifdef DEBUG
DB_EVAL("pawn", "e pawn is blocked")
#endif /* DEBUG */
      }
    }
  }

  /* En finale, */
  if (gs == ENDGAME) {

    /* le programme favorise l'change de matriel s'il possde un bon
       avantage matriel, */
    if (score * cf > 150 && piece_count < start_piece_count) {

      score += bonus_exchange * cf;
#ifdef DEBUG
DB_EVAL("endgame", "bonus for exchange")
#endif /* DEBUG */
    }
    /* dans le cas contraire les changes sont vits. */
    else if (score * cf < 80 && piece_count < start_piece_count) {

      score += malus_exchange * cf;
#ifdef DEBUG
DB_EVAL("endgame", "malus for exchange")
#endif /* DEBUG */
    }
  }

  /* Si le programme possde un net avantage et qu'il n'est pas en manque
     de temps, */
  if (score > 150 && (computer_color ? (uci_btime + 6 * uci_binc) : (uci_wtime
      + 6 * uci_winc)) > (base_time * 6)) {

    /* alors un malus est appliqu pour viter la rptition de la position. */
    score += repeat_counter * malus_repetition;
#ifdef DEBUG
DB_EVAL("endgame", "malus for repetition")
#endif /* DEBUG */
  }

#ifdef DEBUG
debug.print (" : active_color %d - score %d\n",
  active_color, active_color == BLACK ? - score : score);
#endif /* DEBUG */

  return (active_color == BLACK ? - score : score);
}
/* End of function : evaluate_position */

/*******************************************************************************
  Object declaration :
*******************************************************************************/
evaluate_module_s evaluate = {

  check_game_phase,
  evaluate_position
};
