/*******************************************************************************
  File : hash.c

  Description : Module de hachage.
*******************************************************************************/

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte des diffrents modules du programme : */
#include "debug.h"
#include "hash.h"
#include "tools.h"

/*******************************************************************************
  Data :
*******************************************************************************/

/* Valeur de hachage des positions de la partie :
   (Pour vrifier la rptition des position) */
h_value_s h_position;
h_value_s h_position_history[MOVES_LIMIT * 2];

/* Tables des valeurs de hachage pour successivement les pices, la couleur au
   trait, la prise en passant et les roques : */
h_value_s h_values[15][120];
h_value_s h_values_color[2];
h_value_s h_values_ep[120];
h_value_s h_values_castle[5][2];

/*******************************************************************************
  External functions :
*******************************************************************************/

/*******************************************************************************
  Function     : compute_h_position
  Description  : Calcule une valeur de hachage pour la position courante. C'est
                 une valeur numrique unique qui va permettre d'identifier
                 celle-ci.
                 Elle est obtenue en utilisant la fonction logique "ou exclusif"
                 sur des valeurs qui caractrisent les lments suivants :
                    - les pices,
                    - la couleur au trait,
                    - la prise en passant,
                    - la possibilit de roquer.
                 Nota : Un problme important  tudier concerne les collisions
                 qui pourraient apparatre en raison du nombre astronomique des
                 positions possibles.
  Parameters   : aucun
  Return value : identifiant de la position
  Validation   : 1
*******************************************************************************/
h_value_s compute_h_position (void) {

/* ------------------------------- Local data ------------------------------- */

h_value_s h_current_position = { 0, 0 }; /* Valeur d'identification initiale. */

int color;         /* Couleur du camp. */
int i;             /* Indice de boucle. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("compute_h_position\n", false, &compute_h_position_calls);
#endif /* DEBUG */

  /* Prise en compte des pices prsentes sur l'chiquier, de leur couleur et
     de leur position. */
  for (color = BLACK; color <= WHITE; color++) {

    for (i = 1; i <= pieces_number[color]; i++) {

      if (pieces[color][i]) {

        hash.update_h_position (&h_current_position,
          h_values[board[pieces[color][i]]][pieces[color][i]]);
      }
    }
  }

  /* Prise en compte d'une ventuelle prise en passant. */
  hash.update_h_position (&h_current_position, h_values_ep[EP_SQUARE]);

  /* Prise en compte des roques.
    (voir la description de castle_status dans data.c) */
  for (i = 1; i <= 4; i++) {

    hash.update_h_position (&h_current_position,
      h_values_castle[i][(CASTLE_STATUS & castle_data_mask[i]) ? 1 : 0]);
  }

  /* Prise en compte de la couleur au trait. */
  hash.update_h_position (&h_current_position, h_values_color[active_color]);

  return (h_current_position);
}
/* End of function : compute_h_position */

/*******************************************************************************
  Function     : initialize_h_values
  Description  : Calcul et mmorisation des valeurs de hachage pour tous les
                 lments qui permettent de caractriser une position :
                    - les pices,
                    - la couleur au trait,
                    - la prise en passant,
                    - la possibilit de roquer.
                 Chaque valeur est constitue d'un nombre alatoire de 32 bits.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void initialize_h_values (void) {

/* ------------------------------- Local data ------------------------------- */

int color;                   /* Couleur du camp. */
int i;                       /* Indice. */
int piece;                   /* Pice du jeu : pion ou figure. */
int square;                  /* Case de l'chiquier. */
int x, y;                    /* Colonne et range de l'chiquier. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("initialize_h_values\n", true, &init_h_values_calls);
#endif /* DEBUG */

  /* Efface le contenu des tables des valeurs de hachage. */
  memset (h_values, 0, sizeof (h_values));
  memset (h_values_ep, 0, sizeof (h_values_ep));

  for (color = BLACK; color <= WHITE; color++) {

    for (x = A; x <= H; x++) {

      /* Calcul des valeurs de hachage pour les pices : chaque pice aura une
         valeur caractristique en fonction de son type, de sa couleur et de sa
         position sur l'chiquier. */
      for (piece = BPAWN; piece <= BKING; piece++) {

        for (y = 1; y <= 8; y++) {

          square = (y + 1) * 10 + x;

          h_values[piece + (color << 3)][square].h1 = tools.generate_random ();
          h_values[piece + (color << 3)][square].h2 = tools.generate_random ();
        }
      }

      /* Calcul des valeurs de hachage pour chaque case de destination d'une
         prise en passant en fonction de la couleur et de la colonne (cases
         situes sur la troisime range du camp des blancs ou des noirs) : */
      square = (RR(3) + 1) * 10 + x;

      h_values_ep[square].h1 = tools.generate_random ();
      h_values_ep[square].h2 = tools.generate_random ();
    }

    /* Calcul des valeurs de hachage pour la couleur au trait : */
    h_values_color[color].h1 = tools.generate_random ();
    h_values_color[color].h2 = tools.generate_random ();
  }

  /* Calcul des valeurs de hachage pour la prise en compte des roques (petit ou
     grand roque, blanc ou noir) en fonction de sa possibilit : */
  for (i = 1; i <= 4; i++) {

    /* Roque impossible */
    h_values_castle[i][0].h1 = tools.generate_random ();
    h_values_castle[i][0].h2 = tools.generate_random ();

    /* Roque possible */
    h_values_castle[i][1].h1 = tools.generate_random ();
    h_values_castle[i][1].h2 = tools.generate_random ();
  }
}
/* End of function : initialize_h_values */

/*******************************************************************************
  Function     : update_h_position
  Description  : Met  jour la valeur de hachage d'une position avec une valeur
                 de hachage d'un des lments qui entre dans son calcul.
  Parameters   : out - valeur de hachage d'une position
                 in  - valeur de hachage d'un lment
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void update_h_position (h_value_s *new_h_position, h_value_s h_element) {

#ifdef DEBUG
debug.trace ("update_h_position", false, &update_h_position_calls);
#endif /* DEBUG */

  new_h_position->h1 ^= h_element.h1;
  new_h_position->h2 ^= h_element.h2;
}
/* End of function : update_h_position */

/*******************************************************************************
  Module declaration :
*******************************************************************************/
hash_module_s hash = {

  compute_h_position,
  initialize_h_values,
  update_h_position
};
