/*******************************************************************************
  Licence MIT
********************************************************************************
# Copyright (c) 2008/2010 <copyright Christophe J. Mandin>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*******************************************************************************/

/*******************************************************************************
  File : main.c

  Description : Module principal.

                Moteur d'checs gratuit.
*******************************************************************************/

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichier en-tte des donnes gnrales du programme : */
#include "main.h"

/* Fichier en-tte des donnes globales du programme : */
#include "data.h"

/* Fichiers en-tte des diffrents modules du programme : */
#include "book.h"
#include "debug.h"
#include "evaluate.h"
#include "hash.h"
#include "moves.h"
#include "search.h"
#include "tools.h"
#include "uci.h"

/*******************************************************************************
  Macros :
*******************************************************************************/

/* Pour factoriser la lecture des valeurs des paramtres en provenance du
   fichier de configuration : */
#define READ_BOOLEAN_PARAMETER(n,p) \
  if (!strcmp (pointer, n)) { \
    pointer = strtok (NULL, " "); \
    if (!strcmp (pointer, "yes")) { \
      p = true; \
    } \
    else if (!strcmp (pointer, "no")) { \
      p = false; \
    } \
    else { \
      tools.send_output (true, \
        "   ERROR : Unknown value for parameter %s (%s)\n", n, pointer); \
    } \
  }

#define READ_INTEGER_PARAMETER(n,p) \
  if (!strcmp (pointer, n)) { \
    pointer = strtok (NULL, " "); \
    p = atoi (pointer); \
  }

#define READ_STRING_PARAMETER(n,p) \
  if (!strcmp (pointer, n)) { \
    pointer = strtok (NULL, ";"); \
    strcpy (p, pointer); \
  }

/*******************************************************************************
  Data :
*******************************************************************************/

int computer_game_phase = UNDEFINED_GS;

/******************************************************************************
  Internal functions prototypes :
******************************************************************************/

bool call_uci_function (char input[]);
bool execute_command (char input[]);
bool execute_debug_command (char input[]);
void initialize_program (void);

/*******************************************************************************
  External functions :
*******************************************************************************/

/*******************************************************************************
  Function     : initialize_game
  Description  : Met  jour les donnes du programme  partir de la position de
                 dpart donne en entre.
                 Le format de la position (standard FEN/EPD) est dcrit dans le
                 document : "doc\pgn_specification.txt, chapitre 16".
  Parameters   : in  - position de dpart
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void initialize_game (const char string[]) {

/* ------------------------------- Local data ------------------------------- */

char c, *pointer;                 /* Pour parcourir les caractres. */
char position[STRING_LENGTH];     /* Position donne en entre. */

int i, j;                         /* Indices de boucle. */
int square;                       /* Case de l'chiquier. */
int x, y;                         /* Colonne et range de l'chiquier. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("initialize_game\n", true, &init_game_calls);
#endif /* DEBUG */

  /* Par dfaut aucun demi-coup n'a t jou depuis le dbut de la partie. */
  game_ply = 0;

  /* Initialise un chiquier vide de toutes pices. */
  memset (board, 0, sizeof (board));

  /* Le nombre de pices est donc nul. */
  piece_count = 0;

  /* Par dfaut toutes les cases de l'chiquier sont vues comme un bord. */
  for (square = 0; square < 120; square++) {

    board[square] = FRAME;
  }

  /* La chane en entre va tre analyse en dcoupant tous les lments
     identifis dans le format FEN/EPD. */
  strcpy (position, string);

  /* Mise  jour de l'chiquier avec les pices. */
  pointer = strtok (position, " ");
  i = 0;
  c = pointer[i];

  for (y = 8; y >= 1; y--) {

    for (x = A; x <= H; ) {

      square = (y + 1) * 10 + x;

      if (c >= '1' && c <= '8') {

        for (j = 0; j < c - '0'; j++) {

          board[(y + 1) * 10 + x] = NONE;
          x++;
        }
      }
      else {

        switch (c) {
          case 'b': board[square] = BBISHOP; piece_count++; break;
          case 'k': board[square] = BKING;   king[BLACK] = square; break;
          case 'n': board[square] = BKNIGHT; piece_count++; break;
          case 'p': board[square] = BPAWN;   break;
          case 'q': board[square] = BQUEEN;  piece_count++; break;
          case 'r': board[square] = BROOK;   piece_count++; break;
          case 'B': board[square] = WBISHOP; piece_count++; break;
          case 'K': board[square] = WKING;   king[WHITE] = square; break;
          case 'N': board[square] = WKNIGHT; piece_count++; break;
          case 'P': board[square] = WPAWN;   break;
          case 'Q': board[square] = WQUEEN;  piece_count++; break;
          case 'R': board[square] = WROOK;   piece_count++; break;
        }

        x++;
      }
      c = pointer[++i];
    }
    if (y > 1) {

      if (c != '/') {

        tools.send_output (true, "\n   ERROR : Bad position (rank)\n");
      }

      c = pointer[++i];
    }
  }

  /* Mise  jour de la table des pices. */
  tools.update_pieces ();

  /* Dfinition de la couleur qui a le trait. */
  pointer = strtok (NULL, " ");
  if (pointer[0] == 'w') {

    active_color = WHITE;
  }
  else if (pointer[0] == 'b') {

    active_color = BLACK;
  }
  else {

    tools.send_output (true, "\n   ERROR : Bad position (unknown color)\n");
  }

  /* Mise  jour des possibilits de roquer. */
  pointer = strtok (NULL, " ");
  i = 0;
  c = pointer[i];

  if (c == '-') {
  }
  else {
    if (c == 'K') {

      CASTLE_STATUS |= 0x1;
      c = pointer[++i];
    }

   if (c == 'Q') {

      CASTLE_STATUS |= 0x2;
      c = pointer[++i];
    }

    if (c == 'k') {

      CASTLE_STATUS |= 0x4;
      c = pointer[++i];
    }

    if (c == 'q') {

      CASTLE_STATUS |= 0x8;
    }
  }

  /* Mise  jour de la case de prise en-passant ventuelle. */
  pointer = strtok (NULL, " ");
  i = 0;
  c = pointer[i];

  if (c == '-') {

    EP_SQUARE = 0;
  }
  else {

    if (c < 'a' || c > 'h') {

      tools.send_output (true,
        "\n   ERROR : Bad position (file for ep square)");
    }

    y = c - 'a' + 1;
    c = pointer[++i];

    x = c - '1' + 1;

    EP_SQUARE = (y + 1) * 10 + x;
  }

  /* Halfmove clock :
     The fifth field is a nonnegative integer representing the halfmove clock.
     This number is the count of halfmoves (or ply) since the last pawn advance
     or capturing move.  This value is used for the fifty move draw rule. */
  pointer = strtok (NULL, " ");
  c = pointer[0];

  if (!isdigit(c)) {

    tools.send_output (true,
      "\n   ERROR : Bad position (Halfmove is not a digit)");
  }
  else {

    halfmove = atoi (&pointer[0]);
  }

  /* Fullmove number :
     The sixth and last field is a positive integer that gives the fullmove
     number. This will have the value "1" for the first move of a game for
     both White and Black. It is incremented by one immediately after each
     move by Black. */
  pointer = strtok (NULL, " ");
  c = pointer[0];

  if (!isdigit(c)) {

    tools.send_output (true,
      "\n   ERROR : Bad position (Fullmove is not a digit)");
  }
  else {

    fullmove = atoi (&pointer[0]);
  }

  /* Au dpart le programme n'a pas le trait. */
  computer_color = active_color^1;

  /* Le rsultat de la partie est pour le moment indtermin. */
  result = NO_RESULT;

  /* Calcule la valeur de hachage de la position initiale. */
  h_position = hash.compute_h_position ();

  h_position_history[0].h1 = h_position.h1;
  h_position_history[0].h2 = h_position.h2;
}
/* End of function : initialize_game */

/******************************************************************************
  Function     : start_thinking
  Description  : Lancement de la rflexion du programme et traitement du coup
                 choisi, ainsi que d'un ventuel rsultat de la partie.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
******************************************************************************/
void start_thinking (void) {

/* ------------------------------- Local data ------------------------------- */

char lan_move[6];       /* Coup en notation algbrique. */

double elapsed_time_s;  /* En secondes */

int game_phase;    /* Phase de jeu : ouverture, milieu de partie ou finale. */

move_s computer_move;   /* Coup calcul par le programme. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("start_thinking\n", false, &start_thinking_calls);
#endif /* DEBUG */

  /* Identification de la phase de la partie. */
  game_phase = evaluate.check_game_phase ();

  /* Lors du passage d'une phase  une autre, */
  if (computer_game_phase != game_phase) {

    /* alors on affiche la nouvelle phase, */
    if (uci_mode) {

      tools.send_output (true, "info string %s is in the %s.\n", PRODUCT_NAME,
        game_phase == ENDGAME ? "end of game" :
        game_phase == MIDDLE ? "middle of game" : "opening");
    }
    else {

      tools.send_output (true, "\n   %s is in the %s.\n", PRODUCT_NAME,
        game_phase == ENDGAME ? "end of game" :
        game_phase == MIDDLE ? "middle of game" : "opening");
    }

    /* et celle-ci est mmorise. */
    computer_game_phase = game_phase;
  }

  /* Mmorise l'instant de dpart de la rflexion. */
  start_thinking_time = clock ();

  /* Si la recherche dans la bibliothque des ouvertures n'aboutit pas, */
  if (game_phase != OPENING || !book_use || game_ply > BOOK_PLY_LIMIT
      || !book.select_move (&computer_move)) {

    if (!uci_mode) {

      tools.send_output (true, "\n");
    }

    /* alors le programme dmarre sa propre recherche. */
    search_module.think (&computer_move);
  }

#ifdef DEBUG
  debug.close_debug_file ();
#endif /* DEBUG */

  /* Si la partie n'est pas termine, */
  if (   (   (computer_color == BLACK && result != BLACK_IS_MATE)
          || (computer_color == WHITE && result != WHITE_IS_MATE))
      && result != DRAW_FIFTY_MOVES
      && result != DRAW_REPETITION
      && result != DRAW_STALEMATE) {

    /* alors le coup est jou sur l'chiquier. */
    tools.convert_pgm_move (&computer_move, lan_move);
    moves.make (&computer_move);

    /* Si la position courante s'est retrouve trois fois sur l'chiquier, */
    if (tools.is_repeated ()) {

      /* alors la partie est nulle par rptition. */
      result = DRAW_REPETITION;
    }
    /* Si 50 coups ont t jous sans capture ni dplacement de pion, */
    else if (halfmove > 100) {

      /* alors la partie est galement nulle. */
      result = DRAW_FIFTY_MOVES;
    }

    /* Mise  jour de la table des pices. */
    tools.update_pieces ();

    /* Le coup est transmis, */
    if (uci_mode) {

      /* selon le format UCI, */
      tools.send_output (true, "bestmove %s\n", lan_move);
    }
    else {

      /* ou en format standard pour le mode console, */
      tools.send_output (true, "\n   %s plays %s\n", PRODUCT_NAME, lan_move);

      if (nodes) {

        elapsed_time_s = (clock () - start_thinking_time) /
                         (double) CLOCKS_PER_SEC;

        /* avec quelques informations supplmentaires. */
        tools.send_output (true, "\n   Elapsed time %.1f - ", elapsed_time_s);
        tools.send_output (true, "Nodes %d (%d nps)\n", nodes,
          (int) ((float) nodes / elapsed_time_s));
      }

      /* Si l'affichage de l'chiquier est autoris, */
      if (show_board) {

        /* alors il est montr. */
        tools.print_board (stdout);
      }

      /* Si c'est autoris, */
      if (log_use) {

        /* il est galement affich dans le fichier journal. */
        tools.print_board (log_file);
      }
    }
  }

  /* Affichage d'un ventuel rsultat de la partie : */
  if (result) {

    if (uci_mode) {

      tools.send_output (true, "info string ");
    }
    else {

      tools.send_output (true, "\n   ");
    }

    switch (result) {

      case WHITE_IS_MATE :

        tools.send_output (true, "0-1 (black mates)\n");
        break;

      case BLACK_IS_MATE :

        tools.send_output (true, "1-0 (white mates)\n");
        break;

      case DRAW_FIFTY_MOVES :

        tools.send_output (true, "1/2-1/2 (draw - fifty move)\n");
        break;

      case DRAW_REPETITION :

        tools.send_output (true, "1/2-1/2 (draw - 3 repetitions)\n");
        break;

      case DRAW_STALEMATE :

        tools.send_output (true, "1/2-1/2 (draw - stalemate)\n");
        break;

      default :

        tools.send_output (true, "Warning unknown result (%d)\n", result);
        break;
    }
  }
}
/* End of function : start_thinking */

/*******************************************************************************
  Object declaration :
*******************************************************************************/

main_module_s main_module = {

  initialize_game,
  start_thinking
};

/*******************************************************************************
  Internal functions :
*******************************************************************************/

/*******************************************************************************
  Function     : call_uci_function
  Description  : Pour chaque commande UCI reconnue en entre, le programme
                 appelle la fonction du module "uci" correspondante.
  Parameters   : in  - chane de caractres
  Return value : Le paramtre tait une commande UCI (true) ou non (false).
  Validation   : 1
*******************************************************************************/
bool call_uci_function (char input[]) {

/* ------------------------------- Local data ------------------------------- */

bool is_uci_command = true;  /* Indicateur d'une commande UCI. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
  debug.trace ("call_uci_function\n", true, &call_uci_function_calls);
#endif /* DEBUG */

  if (!strncmp (input, "debug", 5)) {

    uci.debug_command ();
  }
  else if (!strncmp (input, "go", 2)) {

    uci.go (input);
  }
  else if (!strcmp (input, "isready")) {

    uci.isready ();
  }
  else if (!strcmp (input, "ponderhit")) {

    uci.ponderhit ();
  }
  else if (!strncmp (input, "position", 8)) {

    uci.position (input);
  }
  else if (!strcmp (input, "quit") || !strcmp (input, "q")) {

    uci.quit (EXIT_SUCCESS);
  }
  else if (!strncmp (input, "register", 8)) {

    uci.registration ();
  }
  else if (!strncmp (input, "setoption", 9)) {

    uci.setoption ();
  }
  else if (!strcmp (input, "stop")) {

    uci.stop ();
  }
  else if (!strcmp (input, "uci")) {

    uci.uci_command ();
  }
  else if (!strcmp (input, "ucinewgame")) {

    uci.ucinewgame ();
  }
  else {

    is_uci_command = false;
  }

  return (is_uci_command);
}
/* End of function : call_uci_function */

/*******************************************************************************
  Function     : execute_command
  Description  : En mode console, traitement de chaque commande connue du
                 programme.
  Parameters   : in  - chane de caractres
  Return value : Le paramtre tait une commande connue (true) ou non (false).
  Validation   : 1
*******************************************************************************/
bool execute_command (char input[]) {

/* ------------------------------- Local data ------------------------------- */

bool is_command = true;  /* Indicateur d'une commande connue en mode console. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
  debug.trace ("execute_command\n", true, &exec_cmd_calls);
#endif /* DEBUG */

  if (!strcmp (input, "board")) {

    show_board = true;
    tools.print_board (stdout);

    if (log_use) {

      tools.print_board (log_file);
    }
  }
  else if (!strcmp (input, "no board")) {

    show_board = false;
    tools.send_output (true, "\n   %s will not show the board.\n", PRODUCT_NAME);
  }
  else if (!strcmp (input, "book")) {

    book_use = true;
    tools.send_output (true, "\n   %s will use book moves.\n", PRODUCT_NAME);
  }
  else if (!strcmp (input, "no book")) {

    book_use = false;
    tools.send_output (true, "\n   %s will not use book moves.\n", PRODUCT_NAME);
  }
  else if (!strcmp (input, "force")) {

    /* Bascule en mode forc ou non. */
    forced_mode ^= 1;

    /* Dans tous les cas, l'ordinateur n'a pas le trait. */
    computer_color = active_color ^1;

    if (forced_mode) {

      tools.send_output (true, "\n   %s is in forced mode.\n", PRODUCT_NAME);
    }
    else {

      tools.send_output (true, "\n   %s is not in forced mode.\n", PRODUCT_NAME);
    }
  }
  else if (!strcmp (input, "game")) {

    /* Affichage des coups de la partie. */


    if (game_ply != 0) {

      tools.send_output (true, "\n");

      int i; for (i = 0; i < game_ply; i++) {

        tools.send_output (true, "%2d %c%d%c%c%d%c\n", i + 1,
          file[played_moves[i].from] + (int) 'a' - 1,
          rank[played_moves[i].from],
          played_moves[i].captured_ptr ? 'x' : '-',
          file[played_moves[i].to] + (int) 'a' - 1,
          rank[played_moves[i].to],
          played_moves[i].status == PROMOTION ?
            played_moves[i].promoted + (int) 'a' - 1 : ' ');
      }
    }
    else {

      tools.send_output (true, "\n   The game list is empty.\n");
    }
  }
  else if (!strcmp (input, "help") || !strcmp (input, "h")) {

    /* Affichage de l'aide. */

  FILE *help_file;
  char help_line[STRING_LENGTH];

    tools.send_output (true, "\n");

    /* Si le fichier d'aide du programme existe, */
    if ((help_file = fopen (HELP_FILE, "r")) == NULL) {

      tools.send_output (true, "ERROR : %s can't open the help file %s\n",
                         PRODUCT_NAME, HELP_FILE);
    }
    else {

      /* alors le programme lit les informations d'aide. */
      while (!feof (help_file)) {

        /* Lecture d'une ligne du fichier de configuration. */
        fgets (help_line, STRING_LENGTH, help_file);

        /* Les lignes des commentaires du fichier ne sont pas affiches. */
        if (help_line[0] != '#' && !feof (help_file)) {

          tools.send_output (true, "%s", help_line);
        }
      }

      fclose (help_file);
    }
  }
  else if (!strcmp (input, "makebook")) {

    /* Construction de la bibliothque d'ouverture. */

    book.make_book ();
  }
  else if (!strcmp (input, "new")) {

    /* Pour jouer une nouvelle partie, on appelle la fontction UCI adquate. */
    uci.ucinewgame ();

    tools.send_output (true, "\n   %s will play a new game.\n", PRODUCT_NAME);

    /* Si l'affichage de l'chiquier est autoris, */
    if (show_board) {

      /* alors il est montr. */
      tools.print_board (stdout);
    }

    /* Si c'est autoris, */
    if (log_use) {

      /* il est galement affich dans le fichier journal. */
      tools.print_board (log_file);
    }
  }
  else if (!strcmp (input, "play")) {

    /* Le programme joue pour la couleur au trait, */
    computer_color = active_color;

    /* et sa rflexion est lance. */
    main_module.start_thinking ();
  }
  else if (!strcmp (input, "quit") || !strcmp (input, "q")) {

    /* Pour quitter une partie, on appelle la fonction UCI adquate. */
    uci.quit (EXIT_SUCCESS);
  }
  else {

    /* Aucune commande ci-dessus n'a t reconnue. */
    is_command = false;
  }

  return (is_command);
}
/* End of function : execute_command */

/*******************************************************************************
  Function     : execute_debug_command
  Description  : En mode console, traitement de chaque commande connue du
                 programme. Les commandes regroupes ici servent  vrifier le
                 bon fonctionnement du programme.
  Parameters   : in  - chane de caractres
  Return value : Le paramtre tait une commande connue (true) ou non (false).
  Validation   : 1
*******************************************************************************/
bool execute_debug_command (char input[]) {

/* ------------------------------- Local data ------------------------------- */

bool is_debug_command = true;     /* Indicateur d'une commande connue. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
  debug.trace ("execute_debug_command\n", true, &exec_dbg_cmd_calls);
#endif /* DEBUG */

  if (!strcmp (input, "dump") || !strcmp (input, "d")) {

    /* Enregistre un tat des donnes du programme : */
    debug.dump ();
  }
  else if (!strcmp (input, "evaluate") || !strcmp (input, "e")) {

#ifdef DEBUG
    debug_evaluation = true;

    if ((debug_eval_file = fopen (DEBUG_EVAL_FILE, "w")) == NULL) {

      tools.send_output (true, "\n   ERROR : %s can't open the debug file %s\n",
              PRODUCT_NAME, DEBUG_EVAL_FILE);
    }
    else {
#endif /* DEBUG */

    tools.send_output (true, "\n   Evaluation : %5.2f\n",
      (float) evaluate.evaluate_position () / 100);

#ifdef DEBUG
    debug_evaluation = false;

    fclose (debug_eval_file);

    tools.send_output (true, "\n   To debug the evaluation, see the file : "
      "%s\n", DEBUG_EVAL_FILE);
    }
#endif /* DEBUG */
  }
  else if (!strcmp (input, "hhistory")) {

    /* Affichage de l'historique des valeurs de hachage des positions. */

  int i = 0;

    tools.send_output (true, "\n   History of hash values for all "
      "positions :\n\n");

    for (i = 0; i <= game_ply; i++) {

      tools.send_output (true, "   h_position_history[%2d] %8x %8x\n",
        i, h_position_history[i].h1, h_position_history[i].h2);
    }
  }
  else if (!strcmp (input, "hposition")) {

    /* Affichage de la valeur de hachage de la position courante. */

  h_value_s current_h_position;

    current_h_position = hash.compute_h_position ();

    tools.send_output (true, "\n   Current hash value %8x %8x\n",
      current_h_position.h1, current_h_position.h2);
  }
  else if (!strcmp (input, "htest")) {

    /* Test pour vrifier les collisions sur les valeurs de hachage. */

    tools.send_output (true, "\n   Please wait, the test is running ...\n");

    debug.test_hash ();
  }
  else if (!strcmp (input, "moves")) {

    /* Donne une liste des coups lgaux dans la position courante avec une
       valuation de cette dernire. */

  int i = 0;
  int legal_moves_number = 0;
  int moves_number = 0;
  move_s moves_list[MOVES_LIMIT];

    tools.send_output (true, "\n");

    moves_number = moves.generate (moves_list, false);

    for (i = 1; i <= moves_number; i++) {

      moves.make (&moves_list[i]);

      if (moves.is_legal (&moves_list[i])) {

        legal_moves_number++;

        tools.send_output (true, "   %c%d%c%c%d%c ",
          file[moves_list[i].from] + (int) 'a' - 1,
          rank[moves_list[i].from],
          moves_list[i].captured_ptr ? 'x' : '-',
          file[moves_list[i].to] + (int) 'a' - 1,
          rank[moves_list[i].to],
          moves_list[i].status == PROMOTION ?
            (moves_list[i].moveid & 0x7F0000) + (int) 'a' - 1 : ' ');

        tools.send_output (true, "%5.2f\n", active_color ?
          (float) evaluate.evaluate_position () / 100 :
          - (float) evaluate.evaluate_position () / 100);
      }

      moves.unmake (&moves_list[i]);
    }

    if (legal_moves_number == 0) {

      tools.send_output (true, "   There's no legal moves.\n");
    }
    else {

      tools.send_output (true, "\n   Number of legal moves : %d\n",
        legal_moves_number);
    }
  }
  else if (!strcmp (input, "test")) {

    /* Effectue un test de performance du programme bas sur une excution
       successive du gnrateur de coups dans la position courante. */

#ifdef DEBUG
    tools.send_output (true, "\n   Not available in debug mode (too long).\n");
#else
  clock_t start_chronometer;
  clock_t stop_chronometer;
  int i, j;
  int moves_number;
  unsigned int nb_tests = 10000000;

  tools.send_output (true, "\n   Please wait, the test is running ...\n");
  tools.send_output (true, "\n   (It will run %d times the moves\n", nb_tests);
  tools.send_output (true, "    generator, and this, 5 times)\n\n");

  move_s moves_list[MOVES_LIMIT];

    for (j = 0; j < 5; j++) {

      start_chronometer = clock ();

      for (i = nb_tests; i > 0; --i) {

        moves_number = moves.generate (moves_list, false);
      }

      stop_chronometer = clock ();

      tools.send_output (true, "   Elapsed time for test : %.2f\n",
        (stop_chronometer - start_chronometer) / (double) CLOCKS_PER_SEC);
    }
#endif /* DEBUG */
  }
  else if (!strcmp (input, "tree")) {

    /* Pour le dbogage de l'arbre de recherche. */

#ifdef DEBUG
    debug_tree = true;

    if ((debug_tree_file = fopen (DEBUG_TREE_FILE, "w")) == NULL) {

      tools.send_output (true, "\n   ERROR : %s can't open the debug tree file"
        " %s\n", PRODUCT_NAME, DEBUG_TREE_FILE);
    }
    else {

      /* Le programme joue pour la couleur au trait, */
      computer_color = active_color;

      /* et sa rflexion est lance. */
      main_module.start_thinking ();

      debug_tree = false;

      fclose (debug_tree_file);

      tools.send_output (true, "\n   To debug the tree search, see the file : "
        "%s\n", DEBUG_TREE_FILE);
    }
#else
    tools.send_output (true, "\n   Only available in debug mode.\n");
#endif /* DEBUG */
  }
  else {

    /* Aucune commande ci-dessus n'a t reconnue. */
    is_debug_command = false;
  }

  return (is_debug_command);
}
/* End of function : execute_debug_command */

/*******************************************************************************
  Function     : initialize_program
  Description  : Initialisation des lments suivants du programme :
                   - environnement systme,
                   - fichier journal,
                   - paramtres lus  partir du fichier de configuration,
                   - variables globales,
                   - module de hachage.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void initialize_program (void) {

/* ------------------------------- Local data ------------------------------- */

FILE *config_file;                     /* Fichier des paramtres. */
char parameter_line[STRING_LENGTH];    /* Ligne du fichier. */
char *pointer;                         /* Pointeur pour parcourir une ligne. */

/* ------------------------------ Function code ----------------------------- */

  /* Pas d'utilisation de tampons sur les flux d'entre et de sortie. */
  setbuf (stdout, NULL);
  setbuf (stdin, NULL);

  /* Pas d'interruption  partir du clavier, le programme se terminera par la
     commande "quit". */
  signal (SIGINT, SIG_IGN);

  /* Initialisation d'une nouvelle squence de nombres pseudo-alatoires. */
  srand(4);

#ifdef DEBUG
  /* Ouverture du fichier utilis pour dboguer le programme. */
  debug.open_debug_file ();

  /* L'ouverture du fichier, faite ci-dessus, permet de tracer en diffr
     l'appel aux fonctions "main" et "initialize_program". */
  debug.trace ("main\n", true, &main_calls);
  debug.trace ("initialize_program\n", true, &init_program_calls);
#endif /* DEBUG */

  /* Si le fichier de configuration du programme existe, */
  if ((config_file = fopen (CONFIG_FILE, "r")) != NULL) {

    /* alors le programme parcourt le fichier. */
    while (!feof (config_file)) {

      /* Lecture d'une ligne du fichier. */
      fgets (parameter_line, STRING_LENGTH, config_file);

      /* Si la ligne n'est pas un commentaire, */
      if (parameter_line[0] != '#') {

        /* alors le caractre de retour  la ligne est supprim, */
        parameter_line[strlen (parameter_line) - 1] = '\0';

        /* un paramtre est lu, */
        pointer = strtok (parameter_line, " ");

        /* et le programme associe sa valeur  une variable. */
             READ_BOOLEAN_PARAMETER("book_use", book_use)
        else READ_BOOLEAN_PARAMETER("log_use", log_use)
        else READ_BOOLEAN_PARAMETER("show_board", show_board)
        else READ_BOOLEAN_PARAMETER("test_mode", test_mode)
        else READ_BOOLEAN_PARAMETER("search_on_depth_limit",
               search_on_depth_limit)

        else READ_INTEGER_PARAMETER("maximum_depth", maximum_depth)
        else READ_INTEGER_PARAMETER("maximum_quiet_depth", maximum_quiet_depth)
        else READ_INTEGER_PARAMETER("minimum_depth", minimum_depth)

        else READ_INTEGER_PARAMETER("maximum_search_time", maximum_search_time)
        else READ_INTEGER_PARAMETER("time_in_console_mode",
               time_in_console_mode)

        else READ_STRING_PARAMETER("start_position", start_position);
      }
    }

    /* Fermeture du fichier de configuration. */
    fclose (config_file);
  }
  else {

    /* sinon le programme ne trouve pas le fichier de configuration, */
    printf ("\n   WARNING : %s can't open the config file %s\n",
      PRODUCT_NAME, CONFIG_FILE);
    printf ("\n   It continues with default values.\n");

    /* Default values for the configuration : */
    book_use              = true;
    log_use               = false;
    search_on_depth_limit = false;
    show_board            = true;
    test_mode             = false;

    maximum_depth        = 20;
    maximum_quiet_depth  = 30;
    minimum_depth        =  5;

    maximum_search_time  = 600000;
    time_in_console_mode =   5000;
  }

  /* Si c'est autoris, */
  if (log_use) {

    /* les changes entre le programme et l'extrieur seront mmoriss dans un
       fichier journal. */
    if ((log_file = fopen (LOG_FILE, "w")) == NULL) {

      /* Le fichier journal ne peut tre ouvert, */
      tools.send_output (true, "\n   ERROR : %s can't open the log file %s\n",
        PRODUCT_NAME, LOG_FILE);

      /* le programme est arrt. */
      uci.quit (EXIT_FAILURE);
    }
  }

  /* Par dfaut, le programme dmarre en mode console (mode UCI non activ). */
  forced_mode = false;
  uci_mode    = false;

  /* Initialisation du coup sans signification. */
  memset (&null_move, 0, sizeof (null_move));

  /* Initialisation du module de hachage. */
  hash.initialize_h_values ();
}
/* End of function : initialize_program */

/*******************************************************************************
  Function     : main
  Description  : Point d'entre du programme.
  Parameters   : in  - nombre des arguments du programme
                 in  - liste des arguments
  Return value : code de sortie du programme
                 (son traitement est report dans la fonction "uci.quit")
  Validation   : 1
*******************************************************************************/
int main (int argc, char *argv[]) {

/* ------------------------------- Local data ------------------------------- */

char input[STRING_LENGTH] = "no input yet"; /* Pour traiter les entres. */

move_s move;                                /* Coup au format interne. */

/* ------------------------------ Function code ----------------------------- */

  /* Initialisation du programme. */
  initialize_program ();

  /* Prsentation du programme en mode console. */
  tools.send_output (true, "\n        %s %s", PRODUCT_NAME, PRODUCT_VERSION);
  tools.send_output (true, "\n        by %s\n", PRODUCT_AUTHOR);
  tools.send_output (true, "\n        Type help to display help !\n");

#ifdef DEBUG
  tools.send_output (true, "\n   **** Debug mode is activated ! ****\n");
#endif /* DEBUG */

  if (test_mode) {

    tools.send_output (true, "\n   **** Test mode is activated ! *****\n");
  }

  /* Initialise une partie avec la position de dpart. */
  main_module.initialize_game (start_position);

  /* Si l'affichage de l'chiquier est autoris, */
  if (show_board) {

    /* alors il est montr. */
    tools.print_board (stdout);
  }

  /* Si c'est autoris, */
  if (log_use) {

    /* il est galement affich dans le fichier journal. */
    tools.print_board (log_file);
  }

  /* Boucle infinie dans l'attente d'une sollicitation extrieure. */
  while (true) {

    /* Si l'ordinateur a le trait, que le mode forc n'est pas activ et
       qu'aucun rsultat n'est dtermin, */
    if (computer_color == active_color && !forced_mode && !result) {

      /* alors la rflexion du programme est lance. */
      main_module.start_thinking ();
    }

    /* En mode console, */
    if (!uci_mode) {

      /* affichage du prompt qui indique l'attente d'une commande. */
      tools.send_output (true, "\n%s -> ", PRODUCT_NAME);
    }

    /* Lecture d'une entre. */
    tools.read_input (input);

    /* Si l'entre peut correspondre  un coup crit en notation algbrique
       (LAN : Long Algebraic Notation), */
    if (   isalpha ((int) input[0])
        && isdigit ((int) input[1])
        && isalpha ((int) input[2])
        && isdigit ((int) input[3])) {

      /* si la conversion identifie un coup possible, */
      if (tools.convert_lan_move (input, &move)) {

        /* et si le coup est lgal dans la position courante, */
        if (tools.verify_move (input)) {

          /* alors il est jou sur l'chiquier, */
          moves.make (&move);

          /* et la table des pices est mise  jour. */
          tools.update_pieces ();

          /* Si l'ordinateur ne joue pas et que l'affichage de l'chiquier est
             autoris, */
          if (show_board) {

            /* alors ce dernier est montr. */
            tools.print_board (stdout);
          }

          /* Si c'est autoris, */
          if (log_use) {

            /* il est galement affich dans le fichier journal. */
            tools.print_board (log_file);
          }
        }
        else {

          /* Le coup est illgal. */
          tools.send_output (true, "\n   ERROR : The move is not legal.\n");
        }
      }
      else {

        /* L'entre ne correspond pas  un coup. */
        tools.send_output (true, "\n   ERROR : This is not a possible move.\n");
      }
    }
    /* S'il ne s'agit pas d'un coup, */
    else {

      /* teste la correspondance avec une commande UCI. */
      if (!call_uci_function (input)) {

        /* teste la correspondance avec toute autre commande connue. */
        if (!execute_command (input) && !execute_debug_command (input)) {

          tools.send_output (true, "\n   ERROR : Unknown input : %s\n", input);
        }
      }
    }
  }
}
/* End of function : main */
