/*******************************************************************************
  File : moves.c

  Description : Module de gnration des coups et de dplacement des pices.
*******************************************************************************/

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte des diffrents modules du programme : */
#include "debug.h"
#include "hash.h"
#include "moves.h"
#include "tools.h"

/*******************************************************************************
  Types :
*******************************************************************************/

/* Informations sur les dplacements possibles d'une pice  : */
typedef struct {

  /* Indique une pice  long rayon d'action : */
  bool long_range;

  /* Liste des directions que peut prendre une pice : */
  int direction[9];

} piece_liberty_s;

/*******************************************************************************
  Constants :
*******************************************************************************/

/* Vecteurs de dplacement dans chacune des 18 directions : */
const int vector[19] = {
  0, /* Non utilis. */

  /* Dplacement d'une case dans toutes les directions : */
  10, 11, 1, -9, -10, -11, -1, 9,

  /* Pour les cavaliers uniquement : */
  21, 12, -8, -19, -21, -12, 8, 19,

  /* Pour les pions uniquement,  partir de leur case de dpart : */
  20, -20 };

/* Directions opposes (utilises pour les 8 premires des 18 existantes) : */
const int opposed_direction[9] = { 0, 5, 6, 7, 8, 1, 2, 3, 4 };

/* Directions autorises pour les pices : */
const piece_liberty_s liberty[15] = {
  { false, { 0,  0,  0,  0,  0,  0,  0,  0,  0 } }, /* not used */

  { false, { 0,  5, 18,  4,  6,  0,  0,  0,  0 } }, /* black pawn */
  { false, { 0,  9, 10, 11, 12, 13, 14, 15, 16 } }, /* black knight */
  {  true, { 0,  0,  2,  0,  4,  0,  6,  0,  8 } }, /* black bishop */
  {  true, { 0,  1,  0,  3,  0,  5,  0,  7,  0 } }, /* black rook */
  {  true, { 0,  1,  2,  3,  4,  5,  6,  7,  8 } }, /* black queen */
  { false, { 0,  1,  2,  3,  4,  5,  6,  7,  8 } }, /* black king */

  { false, { 0,  0,  0,  0,  0,  0,  0,  0,  0 } }, /* not used */
  { false, { 0,  0,  0,  0,  0,  0,  0,  0,  0 } }, /* not used */

  { false, { 0,  1, 17,  2,  8,  0,  0,  0,  0 } }, /* white pawn */
  { false, { 0,  9, 10, 11, 12, 13, 14, 15, 16 } }, /* white knight */
  {  true, { 0,  0,  2,  0,  4,  0,  6,  0,  8 } }, /* white bishop */
  {  true, { 0,  1,  0,  3,  0,  5,  0,  7,  0 } }, /* white rook */
  {  true, { 0,  1,  2,  3,  4,  5,  6,  7,  8 } }, /* white queen */
  { false, { 0,  1,  2,  3,  4,  5,  6,  7,  8 } }  /* white king */
};

/*******************************************************************************
  External functions :
*******************************************************************************/

/*******************************************************************************
  Function     : generate
  Description  : Gnrateur des coups  partir de la position courante et pour
                 la couleur active.
                 La liste des coups fait abstraction des checs. La lgalit de
                 chaque coup sera traite lors du parcours de l'arbre de
                 recherche, ce qui permet de rduire le temps consacr  la
                 gnration.
                 Une option consiste  ne gnrer que les coups qui permettent
                 une capture, ce qui est utile pour la recherche des positions
                 calmes.
  Parameters   : out - liste des coups gnrs
                 in  - recherche uniquement les captures
  Return value : nombre de coups gnrs
  Validation   : 1
*******************************************************************************/
int generate (move_s *moves, bool captures_only) {

/* ------------------------------- Local data ------------------------------- */

int color = active_color;  /* Couleur du camp. */
int i, j;                  /* Indices de boucle. */
int moves_number = 0;      /* Nombre de coups gnrs. */

int square;                /* Case de l'chiquier. */

/* Variables intermdiaires pour gagner du temps en vitant des calculs
   inutiles. En effet  chaque accs  un lment d'un tableau, le programme
   doit calculer l'adresse mmoire de l'lment. */
int DIRECTION;
int PIECES_PIECE;
int PIECES_SQUARE;

move_s *MOVE_PTR;

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("generate", true, &generate_calls);
#endif /* DEBUG */

#ifdef DEBUG
DB_GENERATE_1
#endif /* DEBUG */

  for (i = 1; i <= pieces_number[color]; i++) {

    /* Mmorisent la position de la pice et son type. */
    PIECES_SQUARE = pieces[color][i];
    PIECES_PIECE  = board[PIECES_SQUARE];

    /* Si une pice diffrente d'un pion est prsente, */
    if ((PIECES_PIECE & 0x7) != BPAWN && PIECES_SQUARE) {

      /* alors pour les 8 directions maximum que peut avoir une pice, */
      for (j = 1; j <= 8; ++j) {

        /* le programme mmorise cette direction. */
        DIRECTION = liberty[PIECES_PIECE].direction[j];

        /* Si la pice peut prendre cette direction, */
        if (DIRECTION) {

          /* alors la recherche commence  partir de la position de la pice. */
          square = PIECES_SQUARE;

          do {

            /* Le programme calcule la case suivante dans cette direction. */
            square += vector[DIRECTION];

            /* Si ce n'est pas un bord de l'chiquier, */
            if (board[square] != FRAME) {

              /* alors le programme vrifie si la case contient une pice de
                 la mme couleur ou un roi, */
              if (   (board[square] && ((board[square] >> 3) == color))
                  || ((board[square] & 0x7) == BKING)) {

                /* auquel cas la recherche dans cette direction est termine. */
                break;
              }
              else {

                /* Si la case d'arrive contient une pice adverse ou si la
                   recherche ne se limite pas aux seuls captures, */
                if (!captures_only || board[square]) {

                  /* alors un dplacement est trouv. */
                  moves_number++;

                  MOVE_PTR = &moves[moves_number];

                  /* et les informations de ce dplacement sont mises  jour. */
                  MOVE_PTR->from = PIECES_SQUARE;
                  MOVE_PTR->to = square;
                  MOVE_PTR->status = 0;
                  MOVE_PTR->promoted = 0;
                  MOVE_PTR->moveid = PIECES_SQUARE + (square << 8);

                  /* Si c'est une capture, */
                  if (board[square]) {

                    /* alors le programme mmorise les informations au sujet de
                       la pice capture, */
                    MOVE_PTR->captured = board[square];
                    MOVE_PTR->captured_ptr = pieces_ptr[square];

                    /* et la recherche dans cette direction est termine. */
                    break;
                  }
                  else {

                    /* sinon c'est un simple dplacement. */
                    MOVE_PTR->captured = 0;
                    MOVE_PTR->captured_ptr = NULL;
                  }
                }
              }
            }
            else {

              /* sinon on dpasse les limites de l'chiquier. */
              break;
            }

          /* La recherche dans cette direction continue pour une pice  longue
             porte. */
          } while (liberty[PIECES_PIECE].long_range);
        }
      }
    }
    /* si la pice est un pion, */
    else if ((PIECES_PIECE & 0x7) == BPAWN && PIECES_SQUARE) {

      /* alors pour les 4 directions maximum que peut avoir un pion, */
      for (j = 1; j <= 4; j++) {

        /* le programme mmorise cette direction. */
        DIRECTION = liberty[PIECES_PIECE].direction[j];

        /* et calcule la case suivante dans cette direction. */
        square = PIECES_SQUARE + vector[DIRECTION];

        /* Si ce n'est pas un bord de l'chiquier, */
        if (board[square] != FRAME) {

          /* alors le programme vrifie que le dplacement rpond  l'un des
             critres suivants :
             - la case d'arrive est vide pour un dplacement d'une case,
             - la case d'arrive contient une pice adverse pour une prise,
             - la case d'arrive correspond  une prise en passant,
             - la case d'arrive est vide pour un dplacement de deux cases
               ainsi que la case de passage et que la case de dpart est sur
               la deuxime range. */
          if (   (j == 1 && !board[square])
              || (j > 2 && board[square] && (board[square] >> 3) != color)
              || (j > 2 && EP_SQUARE == square)
              || (j == 2 && !board[square]
                  && !board[square - (color ? 10 : -10)]
                  && rank[PIECES_SQUARE] == RR(2))) {

            /* Si la recherche ne se limite pas aux seuls captures,
               ou si la case d'arrive contient une pice adverse,
               ou si la case d'arrive correspond  une prise en passant,
               ou s'il s'agit d'une promotion,  */
            if (   (!captures_only)
                || (board[square])
                || (j > 2 && EP_SQUARE == square)
                || (rank[square] == RR(8))) {

              /* alors un dplacement est trouv. */
              moves_number++;

              MOVE_PTR = &moves[moves_number];

              /* et les informations de ce dplacement sont mises  jour. */
              MOVE_PTR->from = PIECES_SQUARE;
              MOVE_PTR->to = square;
              MOVE_PTR->moveid = PIECES_SQUARE + (square << 8);

              /* Si c'est une capture, */
              if (board[square]) {

                /* alors le programme mmorise les informations au sujet de
                   la pice capture, */
                MOVE_PTR->captured = board[square];
                MOVE_PTR->captured_ptr = pieces_ptr[square];
              }
              else {

                /* sinon c'est un simple dplacement. */
                MOVE_PTR->captured = 0;
                MOVE_PTR->captured_ptr = NULL;
              }

              /* Si c'est une promotion, */
              if (rank[square] == RR(8)) {

                /* alors le programme mmorise les informations pour la
                   promotion en dame, */
                MOVE_PTR->status = PROMOTION;
                MOVE_PTR->promoted = color ? WQUEEN : BQUEEN;
                MOVE_PTR->moveid = MOVE_PTR->moveid
                  + ((color ? WQUEEN : BQUEEN) << 16);

                /* et crer les sous-promotions associes,
                   en tour, */
                moves_number++;
                memcpy (&moves[moves_number], MOVE_PTR, sizeof(move_s));
                MOVE_PTR->promoted = color ? WROOK : BROOK;
                MOVE_PTR->moveid = MOVE_PTR->moveid
                  + ((color ? WROOK : BROOK) << 16);

                /* en cavalier, */
                moves_number++;
                memcpy (&moves[moves_number], MOVE_PTR, sizeof(move_s));
                MOVE_PTR->promoted = color ? WKNIGHT : BKNIGHT;
                MOVE_PTR->moveid = MOVE_PTR->moveid
                  + ((color ? WKNIGHT : BKNIGHT) << 16);

                /* ou en fou. */
                moves_number++;
                memcpy (&moves[moves_number], MOVE_PTR, sizeof(move_s));
                MOVE_PTR->promoted = color ? WBISHOP : BBISHOP;
                MOVE_PTR->moveid = MOVE_PTR->moveid +
                  + ((color ? WBISHOP : BBISHOP) << 16);
              }
              /* si c'est une prise en passant, */
              else if (j > 2 && EP_SQUARE == square) {

                /* alors le programme met  jour comme pour une capture. */
                MOVE_PTR->status = EN_PASSANT;
                MOVE_PTR->captured = (color ? BPAWN : WPAWN);
                MOVE_PTR->captured_ptr =
                  pieces_ptr[square - (color ? 10 : -10)];
              }
              /* sinon c'est un simple dplacement. */
              else {

                MOVE_PTR->status = 0;
                MOVE_PTR->promoted = 0;
              }
            }
          }
        }
      }
    }
  }

  /* Sauf pour la recherche des seules captures, */
  if (!captures_only) {

    /* le programme examine la possibilit pour un roi de roquer.
       Ce dernier est possible si les cases entre la tour et le roi sont libres.
       Les checs au roi ne seront examins que plus tard dans le module
       "search" pour gagner du temps. */

    /* Examen du petit roque. */
    if (   CASTLE_STATUS & (0x5 & castle_possible_mask[color])
        && !board[96 - 70 * color]
        && !board[97 - 70 * color]) {

      moves_number++;

      MOVE_PTR = &moves[moves_number];

      MOVE_PTR->from = 95 - 70 * color;
      MOVE_PTR->to = MOVE_PTR->from + 2;
      MOVE_PTR->status = KING_CASTLING;
      MOVE_PTR->promoted = 0;
      MOVE_PTR->captured = 0;
      MOVE_PTR->captured_ptr = NULL;
      MOVE_PTR->moveid = MOVE_PTR->from + (MOVE_PTR->to << 8);
    }

    /* Examen du grand roque. */
    if (   CASTLE_STATUS & (0xA & castle_possible_mask[color])
        && !board[94 - 70 * color]
        && !board[93 - 70 * color]
        && !board[92 - 70 * color]) {

      moves_number++;

      MOVE_PTR = &moves[moves_number];

      MOVE_PTR->from = 95 - 70 * color;
      MOVE_PTR->to = MOVE_PTR->from - 2;
      MOVE_PTR->status = QUEEN_CASTLING;
      MOVE_PTR->promoted = 0;
      MOVE_PTR->captured = 0;
      MOVE_PTR->captured_ptr = NULL;
      MOVE_PTR->moveid = MOVE_PTR->from + (MOVE_PTR->to << 8);
    }
  }

#ifdef DEBUG
DB_GENERATE_2
#endif /* DEBUG */

  /* Le nombre de coups trouvs est retourn. */
  return moves_number;
}
/* End of function : generate */

/*******************************************************************************
  Function     : is_attacked
  Description  : Dtermine si une case est attaque par une couleur.
  Parameters   : in  - case
                 in  - couleur attaquante
  Return value : La case est attaque (true) ou non (false).
  Validation   : 1
*******************************************************************************/
bool is_attacked (int square0, int color) {

/* ------------------------------- Local data ------------------------------- */

int d;        /* Direction pour le dplacement autour d'une case. */
int square;   /* Case de l'chiquier. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("is_attacked", true, &is_attacked_calls);
#endif /* DEBUG */

#ifdef DEBUG
debug.print (" : %c%d by %s",
  file[square0] + (int) 'a' - 1, rank[square0], (color ? "white" : "black"));
#endif /* DEBUG */

  /* Pour toutes les directions autour d'une case (sauf les 17 et 18 qui
     correspondent au dplacement de deux cases du pion en dbut de partie), */
  for (d = 1; d < 17; d++) {

    /* On part de la case qui doit tre teste, */
    square = square0;

    while (true) {

      /* et l'on va  la suivante dans la direction dfinie. */
      square += vector[d];

      /* Si la case d'arrive est un bord de l'chiquier,
         ou aprs un saut de cavalier (d > 8) elle est vide,
         ou sur la case d'arrive il y a une pice du mme camp, */
      if (   board[square] == FRAME
          || (!board[square] && d > 8)
          || (board[square] && (board[square] >> 3) != color)) {

        /* alors la recherche dans cette direction est arrte. */
        break;
      }
      /* sinon si la case d'arrive contient une pice du camp adverse, */
      else if (board[square] && (board[square] >> 3) == color) {

        /* alors on regarde si la pice adverse peut aller en direction de la
           case teste : */

        /* Pour les dplacements autres que ceux du cavalier, */
        if (d <= 8) {

          /* si la case contient un pion, */
          if ((board[square] & 0x7) == BPAWN) {

            /* est-ce que le pion peut atteindre la case teste ?  */
            if (   (   liberty[board[square]].direction[3] ==
                       opposed_direction[d]
                    || liberty[board[square]].direction[4] ==
                       opposed_direction[d])
                && (((square - square0) / vector[d]) == 1)) {

#ifdef DEBUG
  debug.print (" = true\n");
#endif /* DEBUG */
              return true;
            }
          }
          /* sinon est-ce que la pice peut atteindre la case teste, en
             fonction de sa distance et de son rayon d'action ? */
          else if (   liberty[board[square]].direction[d] == d
                && (  liberty[board[square]].long_range == true
                    || ((square - square0) / vector[d]) == 1)) {

#ifdef DEBUG
  debug.print (" = true\n");
#endif /* DEBUG */
            return true;
          }
        }
        /* sinon est-ce que la pice est un cavalier. */
        else if ((board[square] & 0x7) == BKNIGHT) {

#ifdef DEBUG
  debug.print (" = true\n");
#endif /* DEBUG */
          return true;
        }

        /* alors la recherche dans cette direction est arrte. */
        break;
      }
    }
  }

#ifdef DEBUG
  debug.print (" = false\n");
#endif /* DEBUG */
  return false;
}
/* End of function : is_attacked */

/*******************************************************************************
  Function     : is_legal
  Description  : Aprs un coup jou, dtermine si le roi est en chec ou si,
                 dans le cas d'un roque, une des cases du roque tait attaque
                 par une pice adverse lors du dplacement du roi.
  Parameters   : in  - coup jou  vrifier
  Return value : Le coup jou tait lgal (true) ou non (false).
  Validation   : 1
*******************************************************************************/
bool is_legal (move_s *move) {

/* ------------------------------- Local data ------------------------------- */

int square;   /* Case de l'chiquier. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("is_legal", false, &is_legal_calls);
#endif /* DEBUG */

  switch (move->status) {

    /* Vrification des checs lors du petit roque. */
    case (KING_CASTLING) :

      square = move->from;

      if (moves.is_attacked (square, active_color)) return false;
      if (moves.is_attacked (square + 1, active_color)) return false;
      if (moves.is_attacked (square + 2, active_color)) return false;
      break;

    /* Vrification des checs lors du grand roque. */
    case (QUEEN_CASTLING) :

      square = move->from;

      if (moves.is_attacked (square, active_color)) return false;
      if (moves.is_attacked (square - 1, active_color)) return false;
      if (moves.is_attacked (square - 2, active_color)) return false;
      break;

    default :
      /* Aprs un coup autre que le roque, vrifie si le roi est en chec. */
      if (moves.is_attacked (king[active_color^1], active_color)) return false;
  }

  /* A ce stade, on peut dire que le dernier coup jou tait lgal. */
  return true;
}
/* End of function : is_legal */

/*******************************************************************************
  Function     : make
  Description  : Joue le coup donn en entre et met  jour les diffrentes
                 donnes du programme.
  Parameters   : in  - coup  jouer
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void make (move_s *move) {

/* ------------------------------- Local data ------------------------------- */

int color;    /* Couleur du camp au trait. */
int i;        /* Indice de boucle. */
int sq0, sq1; /* Pour mmoriser des cases de l'chiquier et ne pas les
                 recalculer. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("make", true, &make_calls);
#endif /* DEBUG */

#ifdef DEBUG
DB_MAKE_1
#endif /* DEBUG */

  color = board[move->from] >> 3;

  /* Mmorise le coup dans la liste des coups jous. */
  memcpy (&played_moves[game_ply], move, sizeof (move_s));

  game_ply++;

  CASTLE_STATUS = castle_status[game_ply - 1];
  EP_SQUARE = ep_square[game_ply - 1];

  /* Mise  jour des informations de hachage pour la position courante. */
  hash.update_h_position (&h_position, h_values[board[move->from]][move->from]);
  hash.update_h_position (&h_position, h_values[board[move->to]][move->to]);
  hash.update_h_position (&h_position, h_values[board[move->from]][move->to]);
  hash.update_h_position (&h_position, h_values_ep[EP_SQUARE]);
  hash.update_h_position (&h_position, h_values_color[BLACK]);
  hash.update_h_position (&h_position, h_values_color[WHITE]);

  /* Prise en compte des possibilits de roquer avant le coup jou. */
  for (i = 1; i <= 4; i++) {

    hash.update_h_position (&h_position,
      h_values_castle[i][(CASTLE_STATUS & castle_data_mask[i]) ? 1 : 0]);
  }

  /* Efface la possibilit de prendre en passant. */
  EP_SQUARE = 0;

  /* Mise  jour du comptage de demi-coups pour la rgle des 50 coups. */
  halfmove_history[search_ply] = halfmove;
  if ((board[move->from] & 0x7) != BPAWN && !move->captured) {

    halfmove++;
  }
  else {

    halfmove = 0;
  }

  /* Mise  jour du dcompte des pices (les pions ne sont pas compts). */
  if ((board[move->to] & 0x7) > BPAWN) {

    piece_count--;
  }

  /* Mise  jour des informations pour la case d'arrive. */
  board[move->to] = board[move->from];
  pieces_ptr[move->to] = pieces_ptr[move->from];
  moved[move->to]++;

  /* Mise  jour de la fiche associe. */
  *pieces_ptr[move->to] = move->to;

  /* Si c'est une capture, */
  if (move->captured) {

    /* alors on efface le lien vers la fiche associe  la pice capture. */
    *move->captured_ptr = 0;
  }

  /* Effacement des informations pour la case de dpart. */
  board[move->from] = NONE;
  moved[move->from]++;

  if ((board[move->to] & 0x7) == BPAWN) {

    /* Si c'est une promotion, la nature de la pice est change. */
    if (move->status == PROMOTION) {

      hash.update_h_position (&h_position,
        h_values[board[move->to]][move->to]);

      /* La pice promue est mise sur l'chiquier. */
      board[move->to] = move->promoted;

      hash.update_h_position (&h_position,
        h_values[move->promoted][move->to]);
    }
    else if (move->status == EN_PASSANT) {

      /* Effacement des informations du pion captur. */
      board[move->to - (color ? 10 : -10)] = NONE;

      hash.update_h_position (&h_position,
        h_values[color ? BPAWN : WPAWN][move->to - (color ? 10 : -10)]);
    }
    else if (abs (move->to - move->from) == 20) {

      /* Positionne la case pour une ventuelle prise en passant. */
      EP_SQUARE = move->from + (color ? 10 : -10);

      hash.update_h_position (&h_position, h_values_ep[EP_SQUARE]);
    }
  }
  /* Si la pice dplace est un roi, */
  else if ((board[move->to] & 0x7) == BKING) {

    /* alors on mmorise sa nouvelle position. */
    king[color] = move->to;

    /* Supprime les possibilits de roquer pour le roi. */
    CASTLE_STATUS &= castle_by_color_mask[color^1];

    /* Si c'est un petit roque la tour 'H' est dplace galement. */
    if (move->status == KING_CASTLING) {

      sq0 = 98 - 70 * color;
      sq1 = sq0 - 2;

      /* Indique que le roi a roqu. */
      CASTLE_STATUS |= castle_done_mask[color];

      /* Mise  jour des informations de la case d'arrive. */
      board[sq1] = (color ? WROOK : BROOK);
      pieces_ptr[sq1] = pieces_ptr[sq0];

      /* Mise  jour de la fiche associe. */
      *pieces_ptr[sq1] = sq1;

      /* Effacement des informations de la case de dpart. */
      board[sq0] = NONE;

      hash.update_h_position (&h_position,
        h_values[color ? WROOK : BROOK][sq0]);
      hash.update_h_position (&h_position,
        h_values[color ? WROOK : BROOK][sq1]);
    }
    /* Si c'est un grand roque la tour 'A' est dplace galement. */
    else if (move->status == QUEEN_CASTLING) {

      sq0 = 91 - 70 * color;
      sq1 = sq0 + 3;

      /* Indique que le roi a roqu. */
      CASTLE_STATUS |= castle_done_mask[color];

      /* Mise  jour des informations de la case d'arrive. */
      board[sq1] = color ? WROOK : BROOK;
      pieces_ptr[sq1] = pieces_ptr[sq0];

      /* Mise  jour de la fiche associe. */
      *pieces_ptr[sq1] = sq1;

      /* Effacement des informations de la case de dpart. */
      board[sq0] = NONE;

      hash.update_h_position (&h_position,
        h_values[color ? WROOK : BROOK][sq0]);
      hash.update_h_position (&h_position,
        h_values[color ? WROOK : BROOK][sq1]);
    }
  }

  /* Mise  jour du statut des roques si une case d'un des coins de l'chiquier
     est concerne. */
  if (move->from == 21 || move->to == 21) {

    /* Le grand roque blanc n'est plus possible. */
    CASTLE_STATUS &= 0x3D;
  }
  if (move->from == 91 || move->to == 91) {

    /* Le grand roque noir n'est plus possible. */
    CASTLE_STATUS &= 0x37;
  }
  if (move->from == 28 || move->to == 28) {

    /* Le petit roque blanc n'est plus possible. */
    CASTLE_STATUS &= 0x3E;
  }
  if (move->from == 98 || move->to == 98) {

    /* Le petit roque noir n'est plus possible. */
    CASTLE_STATUS &= 0x3B;
  }

  /* Prise en compte des possibilits de roquer aprs le coup jou. */
  for (i = 1; i <= 4; i++) {

    hash.update_h_position (&h_position,
      h_values_castle[i][(CASTLE_STATUS & castle_data_mask[i]) ? 1 : 0]);
  }

  h_position_history[game_ply].h1 = h_position.h1;
  h_position_history[game_ply].h2 = h_position.h2;

  /* Mise  jour du numro du coup de la partie. */
  if (active_color == BLACK) {

    fullmove++;
  }

  /* Le trait passe  la couleur oppose. */
  active_color ^= 1;

#ifdef DEBUG
DB_MAKE_2
#endif /* DEBUG */
}
/* End of function : make */

/*******************************************************************************
  Function     : unmake
  Description  : Revient  la position prcdente sur l'chiquier.
  Parameters   : in  - coup  enlever
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void unmake (move_s *move) {

/* ------------------------------- Local data ------------------------------- */

int color;    /* Couleur du camp au trait. */
int sq0, sq1; /* Pour mmoriser des cases de l'chiquier et ne pas les
                 recalculer. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("unmake", true, &unmake_calls);
#endif /* DEBUG */

  if (game_ply) {

    game_ply--;

#ifdef DEBUG
DB_UNMAKE
#endif /* DEBUG */

    color = board[move->to] >> 3;

    h_position.h1 = h_position_history[game_ply].h1;
    h_position.h2 = h_position_history[game_ply].h2;

    /* Mise  jour du comptage de demi-coups pour la rgle des 50 coups. */
    halfmove = halfmove_history[search_ply];

    /* Mise  jour des informations de la case de dpart. */
    board[move->from] = board[move->to];
    pieces_ptr[move->from] = pieces_ptr[move->to];

    /* Mise  jour de la fiche associe. */
    *pieces_ptr[move->from] = move->from;

    /* Enlve l'indication de dplacement de la pice. */
    moved[move->from]--;
    moved[move->to]--;

    /* Effacement des informations de la case d'arrive. */
    if (!move->captured) {

      board[move->to] = NONE;
    }
    else {

      /* Si c'tait une prise en passant. */
      if (move->status == EN_PASSANT) {

        /* Et la fiche associe est mise  jour. */
        *move->captured_ptr = move->to - (color ? 10 : -10);

        /* Le pion captur est remis sur l'chiquier. */
        board[move->to] = NONE;
        board[*move->captured_ptr] = color ? BPAWN : WPAWN;
        pieces_ptr[*move->captured_ptr] = move->captured_ptr;
      }
      else {

        /* La pice capture est remise sur l'chiquier. */
        board[move->to] = move->captured;
        pieces_ptr[move->to] = move->captured_ptr;

        /* Mise  jour du dcompte des pices (les pions ne sont pas compts).*/
        if ((board[move->to] & 0x7) > BPAWN) {

          piece_count++;
        }

        /* Et la fiche associe est mise  jour. */
        *move->captured_ptr = move->to;
      }
    }

    /* Si la pice dplace tait un roi, */
    if ((board[move->from] & 0x7) == BKING) {

      /* alors on mmorise sa nouvelle position. */
      king[color] = move->from;

      /* Si c'tait un petit roque la tour 'H' est dplace galement. */
      if (move->status == KING_CASTLING) {

        sq0 = 98 - 70 * color;
        sq1 = sq0 - 2;

        /* Mise  jour des informations de la case d'arrive. */
        board[sq0] = color ? WROOK : BROOK;
        pieces_ptr[sq0] = pieces_ptr[sq1];

        /* Indique que la tour n'a pas boug. */
        moved[sq0] = 0;

        /* Mise  jour de la fiche associe. */
        *pieces_ptr[sq0] = sq0;

        /* Effacement des informations de la case de dpart. */
        board[sq1] = NONE;
      }

      /* Si c'tait un grand roque la tour 'A' est dplace galement. */
      if (move->status == QUEEN_CASTLING) {

        sq0 = 91 - 70 * color;
        sq1 = sq0 + 3;

        /* Mise  jour des informations de la case d'arrive. */
        board[sq0] = color ? WROOK : BROOK;
        pieces_ptr[sq0] = pieces_ptr[sq1];

        /* Indique que la tour n'a pas boug. */
        moved[sq0] = 0;

        /* Mise  jour de la fiche associe. */
        *pieces_ptr[sq0] = sq0;

        /* Effacement des informations de la case de dpart. */
        board[sq1] = NONE;
      }
    }
    /* Si c'tait une promotion, la nature de la pice est change. */
    else if (move->status == PROMOTION) {

      board[move->from] = color ? WPAWN : BPAWN;
    }

    /* Mise  jour du numro du coup de la partie. */
    if (active_color == WHITE) {

      fullmove--;
    }

    /* Le trait passe  la couleur oppose. */
    active_color ^= 1;
  }
  else {

    tools.send_output (true, "   ERROR : There's no move to undo\n\n");
  }
}
/* End of function : unmake */

/*******************************************************************************
  Object declaration :
*******************************************************************************/

moves_module_s moves = {

  generate,
  is_attacked,
  is_legal,
  make,
  unmake
};
