/*******************************************************************************
  File : search.c

  Description : Module de recherche.
*******************************************************************************/

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte des diffrents modules du programme : */
#include "book.h"
#include "debug.h"
#include "evaluate.h"
#include "hash.h"
#include "moves.h"
#include "search.h"
#include "tools.h"
#include "uci.h"

/*******************************************************************************
  Types :
*******************************************************************************/

typedef struct {

  int score1;
  int score2;

  int moveid1;
  int moveid2;
  int moveid3;

} killer_move_s;

/*******************************************************************************
  Data :
*******************************************************************************/

/* Indique si la recherche doit continuer dans le cas d'une chute du score. */
bool continue_to_search;

/* Mmorise le fait que la recherche doit s'arrter sur manque de temps. */
bool end_of_time;

/* Indique que le score diminue dangereusement par rapport au dernier coup. */
bool is_score_falling;

/* Pour privilgier la recherche de la variante principale. */
bool search_pv;

/* Indique que la recherche est interrompue sans qu'un coup ne soit trouv. */
bool search_failure;

/* Pour mmoriser les coups qui ont provoqu un lagage de l'arbre (rupture sur
   la borne alpha). Indic sur les cases de dpart et d'arrive. */
int history_heuristic[120][120];

/* Pour mmoriser le score du dernier coup slectionn. */
int selected_move_score;

/* Coups remarquables principalement  l'origine d'un lagage beta. */
killer_move_s killers[PV_LIMIT];

/*******************************************************************************
  Internal functions prototypes :
*******************************************************************************/

void apply_heuristic (move_s *move);
int pick_move (move_s *moves_list, int moves_number);
int quiescence_search (int live_depth, int alpha, int beta);
int search (int live_depth, int alpha, int beta);
void sort_moves (move_s *moves_list, int moves_number);
void update_killers (int moveid, int score);

/*******************************************************************************
  External functions :
*******************************************************************************/

/*******************************************************************************
  Function     : think
  Description  : Initialisation de la rflexion du programme.
  Parameters   : out - coup calcul par le programme
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void think (move_s *move) {

/* ------------------------------- Local data ------------------------------- */

int i;             /* Indice de boucle. */
int score;         /* Score remont par l'analyse pour une profondeur donne. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("think\n", true, &think_calls);
#endif /* DEBUG */

  /* Le nombre des noeuds calculs est remis  zro. */
  nodes = 0;
  qnodes = 0;

#ifdef DEBUG
  debug_file_indice = 1;
#endif /* DEBUG */

  /* Autorise  poursuivre la recherche si le score devient trop mauvais. */
  continue_to_search = true;

  /* Calcule le temps allou  la rflexion du programme. */
  tools.compute_time ();

  /* Efface la prcdente variante principale. */
  memset (pv, 0, sizeof (pv));

  /* Efface la prcdente heuristique. */
  memset (history_heuristic, 0, sizeof (history_heuristic));

  /* Efface les prcdents coups remarquables. */
  for (i = 0; i < PV_LIMIT; i++) {

    killers[i].score1 = -MIN_MAX_LIMIT;
    killers[i].score2 = -MIN_MAX_LIMIT;

    killers[i].moveid1 = 0;
    killers[i].moveid2 = 0;
    killers[i].moveid3 = 0;
  }

  /* Mmorise le nombre de pices prsentes sur l'chiquier au dbut de la
     recherche. */
  start_piece_count = piece_count;

  /* La recherche commence en augmentant progressivement la profondeur
     de rflexion, la limite se fera principalement sur le temps. */
  for (depth = 1; depth <= maximum_depth; depth++) {

    /* Calcul du temps coul depuis le dbut de la rflexion. */
    elapsed_time_ms = (int) ((clock () - start_thinking_time) * 1000 /
                             (double) CLOCKS_PER_SEC);

    /* Si le temps pass est trop important, */
    if (   elapsed_time_ms > time_for_move
        && depth > minimum_depth) {

      /* alors la rflexion du programme est interrompue. */
      break;
    }

    /* Pour le moment le programme n'est pas en manque de temps. */
    end_of_time  = false;
    search_failure = false;

    /* Initialise l'indicateur de chute du score. */
    is_score_falling = false;

    /* Initialise le nombre de demi-coups dans la descente de l'arbre. */
    search_ply = 1;

    /* Favorise la recherche de la variante principale. */
    search_pv = true;

    /* Lance la recherche  la racine de l'arbre. */
    score = search (depth, -MIN_MAX_LIMIT, MIN_MAX_LIMIT);

    /* Si la recherche n'a pas t infructueuse par manque de temps, */
    if (!search_failure) {

      /* alors le coup trouv est celui  prendre en comtpe pour le moment. */
      *move = pv[1][1];

      /* Mmorise le score du coup slectionn. */
      selected_move_score = score;

      /* Si la rflexion est suffisamment pertinente, */
      if (depth >= minimum_depth) {

        /* alors le programme affiche des informations sur l'tat de la
           recherche. */
        uci.info (score);
      }
    }

    /* Les scores des coups remarquables sont inutilisables pour les profondeurs
       suprieures, mais les coups peuvent servir pour l'algorithme de tri. */
    for (i = 0; i < PV_LIMIT; i++) {

      killers[i].score1 = -MIN_MAX_LIMIT;
      killers[i].score2 = -MIN_MAX_LIMIT;
    }
  }

#ifdef DEBUG
DB_TREE_1
#endif /* DEBUG */
}
/* End of function : think */

/*******************************************************************************
  Object declaration :
*******************************************************************************/

search_module_s search_module = {

  think
};

/*******************************************************************************
  Internal functions :
*******************************************************************************/

/*******************************************************************************
  Function     : apply_heuristic
  Description  : Cette fonction teste, pour le coup en entre, si une
                 heuristique base sur les coups remarquables (killers) ou sur
                 un historique des coupures alpha et beta peut s'appliquer.
  Parameters   : in  - coup  analyser
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void apply_heuristic (move_s *move) {

#ifdef DEBUG
debug.trace ("apply_heuristic", false, &apply_heuristic_calls);
#endif /* DEBUG */

  /* Le programme favorise les coups remarquables trouvs au mme niveau de
     rflexion. */
  if (move->moveid == killers[search_ply].moveid1) {

    move->note += 900;

#ifdef DEBUG
debug.print (" (1.4 %d)", move->note);
#endif /* DEBUG */
  }
  else if (move->moveid == killers[search_ply].moveid2) {

    move->note += 600;

#ifdef DEBUG
debug.print (" (1.5 %d)", move->note);
#endif /* DEBUG */
  }
  else if (move->moveid == killers[search_ply].moveid3) {

    move->note += 300;

#ifdef DEBUG
debug.print (" (1.6 %d)", move->note);
#endif /* DEBUG */
  }

  /* Le programme favorise enfin les coups qui ont provoqu des coupures alpha
     ou beta en se basant uniquement sur les cases de dpart et d'arrive.
     Par contre la valeur de la note est inversement proportionnelle  la
     profondeur atteinte dans la recherche. */
  move->note += history_heuristic[move->from][move->to] >> depth;

#ifdef DEBUG
debug.print (" (1.7 %d)", move->note);
#endif /* DEBUG */
}
/* End of function : apply_heuristic */

/*******************************************************************************
  Function     : pick_move
  Description  : A chaque appel, retourne le meilleur coup  analyser, suite au
                 tri effectu sur la liste des coups possibles dans une position
                 donne.
  Parameters   : in  - liste des coups possibles
                 in  - nombre des coups de cette liste
  Return value : numro du meilleur coup dans la liste en entre
  Validation   : 1
*******************************************************************************/
int pick_move (move_s *moves_list, int moves_number) {

/* ------------------------------- Local data ------------------------------- */

int best_note = -1; /* Pour ranger la meilleure note des coups de la liste; la
                       valeur -1 indique que le coup n'est plus  prendre en
                       compte pour le prochain appel  la fonction. */
int i;              /* Indice de boucle. */
int move = 0;       /* Le coup 0 n'existe pas, il indiquera la fin de la
                       recherche. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("pick_move", true, &pick_move_calls);
#endif /* DEBUG */

  /* Pour chaque coup de la liste, */
  for (i = 1; i <= moves_number; i++) {

    /* s'il est plus intressant que le prcdent, */
    if (moves_list[i].note > best_note) {

      /* alors il est slectionn. */
      best_note = moves_list[i].note;
      move = i;
    }
  }

  /* Pour viter que le coup soit  nouveau slectionn au prochain appel. */
  moves_list[move].note = -1;

#ifdef DEBUG
debug.print (" : move %d\n", move);
#endif /* DEBUG */

  return move;
}
/* End of function : pick_move */

/*******************************************************************************
  Function     : quiescence_search
  Description  : Recherche d'une position calme qui permette une valuation.
  Parameters   : in  - profondeur de rflexion instantanne
                 in  - borne alpha
                 in  - borne beta
  Return value : valuation de la position atteinte
  Validation   : 1
*******************************************************************************/
int quiescence_search (int live_depth, int alpha, int beta) {

/* ------------------------------- Local data ------------------------------- */

bool legal_move;                /* Indique si le coup est lgal ou non. */

int i, j;                       /* Indice de boucle. */
int initial_score;
int legal_moves_found = 0;      /* Nombre des coups lgaux de la position. */
int moves_number;               /* Nombre des coups possibles d'une position. */
int score = -MIN_MAX_LIMIT;     /* Score de la branche analyse. */

move_s moves_list[MOVES_LIMIT]; /* Pour ranger les coups d'une position. */

#ifdef DEBUG
int debug_node;
#endif /* DEBUG */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("quiescence_search", true, &quiescence_search_calls);
#endif /* DEBUG */

#ifdef DEBUG
debug.print (" : live_depth %d alpha %d beta %d\n", live_depth, alpha, beta);
#endif /* DEBUG */

  /* Evaluation de la position courante  l'appel de la fonction : */
  initial_score = evaluate.evaluate_position ();

  /* Si la profondeur maximum est atteinte ou si il y a une coupure beta : */
  if (!live_depth || initial_score >= beta) {

    /* alors on remonte directement l'valuation obtenue. */
    return initial_score;
  }

  /* La longueur de la variante principale augmente. */
  pv_length[search_ply] = search_ply;

  /* Si l'valuation est suprieure  la prcdente, */
  if (initial_score > alpha) {

    /* alors la nouvelle valuation devient la rfrence. */
    alpha = initial_score;
  }

  /* Gnration des seules captures possibles dans la position courante. */
  moves_number = moves.generate (moves_list, true);

  /* Tri des coups pour analyser en premier les plus intressants. */
  sort_moves (moves_list, moves_number);

  /* Parcours des captures possibles dans l'ordre du tri prcdent. */
  while ((i = pick_move (moves_list, moves_number))) {

#ifdef DEBUG
DB_TREE_2('q', maximum_quiet_depth - live_depth + 1)
#endif /* DEBUG */

    /* Le coup considr est jou sur l'chiquier, */
    moves.make (&moves_list[i]);
    search_ply++;

    /* et s'il est lgal, */
   if (moves.is_legal (&moves_list[i])) {

      /* un nouveau noeud de l'arbre est atteint. */
      nodes++;
      qnodes++;

#ifdef DEBUG
debug.split_debug_file ();
#endif /* DEBUG */

      /* et l'analyse est poursuivie au niveau suivant de l'arbre. */
      score = - quiescence_search (live_depth - 1, - beta, - alpha);

      /* Maintenant le coup est valid. */
      legal_moves_found++;
      legal_move = true;
    }
    else {

      legal_move = false;
    }

    /* La position prcdente est replace sur l'chiquier. */
    search_ply--;
    moves.unmake (&moves_list[i]);

#ifdef DEBUG
DB_TREE_3
#endif /* DEBUG */

    /* Si l'valuation est suprieure  la prcdente, */
    if (legal_move && score > alpha) {

      /* alors le programme vrifie les coups remarquables, */
      update_killers (moves_list[i].moveid, score);

      /* et s'il y a une coupure beta, */
      if (score >= beta) {

        /* Rien ne sert de continuer  analyser les autres coups situs
           au mme niveau de profondeur (lagage beta). */
        return beta;
      }

      /* La nouvelle valuation devient la rfrence. */
      alpha = score;

      /* Mise  jour de la variante principale. */
      pv[search_ply][search_ply] = moves_list[i];
      for (j = search_ply + 1; j < pv_length[search_ply + 1]; j++) {

        pv[search_ply][j] = pv[search_ply + 1][j];
      }
      pv_length[search_ply] = pv_length[search_ply + 1];
    }
  }

  /* Si aucune capture n'existe  ce niveau de profondeur, */
  if (!legal_moves_found) {

    /* alors on remonte l'valuation de la position courante. */
    return initial_score;
  }
  else {

    /* sinon on remonte le meilleur score de l'arbre. */
    return alpha;
  }
}
/* End of function : quiescence_search */

/*******************************************************************************
  Function     : search
  Description  : Cette fonction est appele rcursivement pour parcourir l'arbre
                 des positions et remonter l'valuation de chacune.
  Parameters   : in  - profondeur de rflexion instantanne
                 in  - borne alpha
                 in  - borne beta
  Return value : nouvelle borne alpha
  Validation   : 1
*******************************************************************************/
int search (int live_depth, int alpha, int beta) {

/* ------------------------------- Local data ------------------------------- */

bool legal_move;                /* Indique si le coup est lgal ou non. */

int i, j;                       /* Indice de boucle. */
int legal_moves_found = 0;      /* Nombre des coups lgaux de la position. */
int moves_number;               /* Nombre des coups possibles d'une position. */
int score = -MIN_MAX_LIMIT;     /* Score de la branche analyse. */
int search_extension = 0;       /* Poursuite de la recherche en cas d'chec. */

move_s moves_list[MOVES_LIMIT]; /* Pour ranger les coups d'une position. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("search", true, &search_calls);
#endif /* DEBUG */

#ifdef DEBUG
debug.print (" : live_depth %d alpha %d beta %d game_ply %d\n",
             live_depth, alpha, beta, game_ply);
#endif /* DEBUG */

  /* Si la profondeur de rflexion est significative, tous les 2**14 noeuds, */
  if (search_ply > 1 && depth > minimum_depth && !(nodes & 16383)) {

    /* alors le temps coul depuis le dbut de la rflexion est calcul. */
    elapsed_time_ms = (int) ((clock () - start_thinking_time) * 1000 /
                            (double) CLOCKS_PER_SEC);

    /* Si le temps octroy pour le coup est dpass, */
    if (elapsed_time_ms > time_for_move) {

      /* sauf dans le cas o le score du programme chute dangereusement, */
      if (is_score_falling && continue_to_search) {

        /* auquel cas, si le temps restant est suffisant, */
        if ((computer_color ? uci_wtime : uci_btime) > (5 * time_for_move)) {

          /* la rflexion sera poursuivie, */
          time_for_move *= 2;

          /* une seule fois. */
          continue_to_search = false;
        }
        else {

          /* sinon la rflexion du programme sera interrompue. */
          end_of_time = true;
          return 0;
        }
      }
      else {

        /* sinon la rflexion du programme sera interrompue. */
        end_of_time = true;
        return 0;
      }
    }
  }

  /* Si la position courante se retrouve pour la troisime fois depuis le dbut
     de la partie, */
  if (tools.is_repeated ()) {

    /* s'il s'agit du premier niveau, */
    if (search_ply == 1) {

      /* le rsultat de la partie est nul, */
      result = DRAW_REPETITION;

      /* la longueur de la variante principale est mise  zro, */
      pv_length[search_ply] = 0;

      /* aucun coup ne sera slectionn. */
      pv[1][1] = null_move;
    }

    /* et le score remont est nul. */
    return 0;
  }

  /* Dfinit la longueur de la variante principale qui augmente  chaque
     niveau de l'arbre. */
  pv_length[search_ply] = search_ply;

  /* Si le roi est attaqu, tout en vitant l'chec perptuel, */
  if (   moves.is_attacked (king[active_color], active_color ^1)
      && search_ply <= maximum_depth) {

    /* alors la recherche sera prolonge. */
    search_extension++;
  }

  /* Si la profondeur de rflexion atteint la limite, */
  if (search_ply > 1 && !(live_depth + search_extension)) {

    /* alors le programme recherche une situation calme, */
    /* avant de remonter l'valuation de la position obtenue. */
    return quiescence_search (maximum_quiet_depth, alpha, beta);
  }

  /* Gnration des coups possibles dans la position courante. */
  moves_number = moves.generate (moves_list, false);

  /* Tri des coups pour analyser en premier les plus intressants. */
  sort_moves (moves_list, moves_number);

  /* Parcours des coups possibles dans l'ordre du tri prcdent. */
  while ((i = pick_move (moves_list, moves_number))) {

#ifdef DEBUG
DB_TREE_2('s', depth - live_depth + 1)
#endif /* DEBUG */

    /* Le coup considr est jou sur l'chiquier, */
    moves.make (&moves_list[i]);
    search_ply++;

    /* et s'il est lgal, */
    if (moves.is_legal (&moves_list[i])) {

      /* un nouveau noeud de l'arbre est atteint. */
      nodes++;

#ifdef DEBUG
debug.split_debug_file ();
#endif /* DEBUG */

      /* et l'analyse est poursuivie au niveau suivant de l'arbre. */
      score = -search (live_depth - 1 + search_extension, -beta, -alpha);

      /* Si la rflexion a t interrompue au premier niveau avant qu'une
         branche ne soit analyse compltement, */
      if (search_ply == 2 && end_of_time && !legal_moves_found) {

        /* alors une alerte est leve. */
        search_failure = true;
      }

      /* Maintenant le coup est valid. */
      legal_move = true;
      legal_moves_found++;
    }
    else {

      legal_move = false;
    }

    /* La position prcdente est replace sur l'chiquier. */
    search_ply--;
    moves.unmake (&moves_list[i]);

#ifdef DEBUG
DB_TREE_3
#endif /* DEBUG */

    /* Si le temps octroy pour la rflexion est dpass, */
    if (end_of_time) {

      if (search_ply == 1) {

        /* alors on remonte le meilleur coup trouv pour le moment. */
        return alpha;
      }
      else {

        /* alors on arrte l'analyse. */
        return 0;
      }
    }

    /* Si l'valuation est suprieure  la prcdente, */
    if (legal_move && score > alpha) {

      /* alors l'intrt du coup trouv augmente pour un prochain tri. */
      history_heuristic[moves_list[i].from][moves_list[i].to] += live_depth;

#ifdef DEBUG
DB_HISTORY_HEURISTIC("search");
#endif /* DEBUG */

      /* Si le score courant chute dangereusement par rapport au score
         du dernier coup slectionn, */
      if (search_ply == 1 && score < (selected_move_score - 50)) {

        /* alors l'information est mmorise pour ventuellement continuer la
           recherche. */
        is_score_falling = true;
      }

      /* Le programme vrifie les coups remarquables. */
      update_killers (moves_list[i].moveid, score);

      /* S'il y a une coupure beta, */
      if (search_ply > 1 && score >= beta) {

        /* Rien ne sert de continuer  analyser les autres coups situs
           au mme niveau de profondeur (lagage beta). */
        return beta;
      }

      /* La nouvelle valuation devient la rfrence. */
      alpha = score;

      /* Mise  jour de la variante principale. */
      pv[search_ply][search_ply] = moves_list[i];
      for (j = search_ply + 1; j < pv_length[search_ply + 1]; j++) {

        pv[search_ply][j] = pv[search_ply + 1][j];
      }
      pv_length[search_ply] = pv_length[search_ply + 1];

      /* Si la rflexion est suffisamment pertinente, */
      if (search_ply == 1 && depth >= minimum_depth) {

        /* alors le programme affiche des informations sur l'tat de la
           recherche. */
        uci.info (alpha);
      }
    }
  }

  /* Si aucun coup lgal n'est trouv, */
  if (!legal_moves_found) {

    /* et si le roi est attaqu par la couleur adverse, */
    if (moves.is_attacked (king[active_color], active_color^1)) {

      /* au premier niveau de l'arbre, */
      if (search_ply == 1) {

        /* le rsultat de la partie est indiqu, */
        result = active_color ? WHITE_IS_MATE : BLACK_IS_MATE;
      }

      /* dans tous les cas le score remont indique le nombre de demi-coups
         avant le mat, */
      alpha = - MIN_MAX_LIMIT + search_ply;
    }
    else {

      /* au premier niveau de l'arbre, */
      if (search_ply == 1) {

        /* le rsultat de la partie est indiqu. */
        result = DRAW_STALEMATE;
      }

      /* dans tous les cas le score remont est nul. */
      alpha = 0;
    }
  }
  else {

    /* Si la rgle des 50 coups s'applique, */
    if (halfmove >= 100) {

      /* au premier niveau de l'arbre, */
      if (search_ply == 1) {

        /* le rsultat de la partie est indiqu, */
        result = DRAW_FIFTY_MOVES;

        /* la longueur de la variante principale est mise  zro, */
        pv_length[search_ply] = 0;
      }

      /* dans tous les cas le score nul est remont. */
      alpha = 0;
    }
  }

  /* Cherche si l'adversaire du programme est mat, */
  if (search_ply == 1 && score == MIN_MAX_LIMIT - 2) {

    /* pour positionner le rsultat de la partie. */
    result = active_color ? BLACK_IS_MATE : WHITE_IS_MATE;
  }

  return alpha;
}
/* End of function : search */

/*******************************************************************************
  Function     : sort_moves
  Description  : Cette fonction fait le tri des coups, pour lancer l'analyse en
                 priorit sur les coups les plus intressants, ceci afin de
                 provoquer un lagage au plus tt.
  Parameters   : in  - liste des coups d'une position
                 in  - nombre de coups dans la liste
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void sort_moves (move_s *moves_list, int moves_number) {

/* ------------------------------- Local data ------------------------------- */

int i;             /* Indice de boucle. */

move_s *MOVE_PTR;  /* Pour gagner du temps en vitant des calculs inutiles. En
                      effet  chaque accs  un lment d'un tableau, le
                      programme doit calculer l'adresse mmoire de l'lment. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("sort_moves", true, &sort_moves_calls);
#endif /* DEBUG */

  /* Pendant la recherche de la variante principale, les coups correspondants
     auront une note plus importante : */
  if (search_pv) {

    search_pv = false;

    for (i = 1; i <= moves_number; i++)   {

      MOVE_PTR = &moves_list[i];

      /* Le programme favorise les captures, */
      if (MOVE_PTR->captured) {

        /* en tenant compte de l'cart entre les valeurs des pices. */
        MOVE_PTR->note =   pieces_value[MOVE_PTR->captured & 0x7]
                         - pieces_value[board[MOVE_PTR->from] & 0x7] + 1000;

#ifdef DEBUG
debug.print (" (1.1 %d)", MOVE_PTR->note);
#endif /* DEBUG */
      }
      else {

        MOVE_PTR->note = 0;

#ifdef DEBUG
debug.print (" (1.2 %d)", MOVE_PTR->note);
#endif /* DEBUG */
      }

      /* Si le coup fait partie de la variante principale, */
      if (MOVE_PTR->moveid == pv[1][search_ply].moveid) {

        /* alors son intrt est trs important. */
        MOVE_PTR->note += MIN_MAX_LIMIT;

#ifdef DEBUG
debug.print (" (1.3 %d)", MOVE_PTR->note);
#endif /* DEBUG */
        /* et le programme poursuit sur la suite de la variante principale. */
        search_pv = true;
      }
      else {

        /* Sinon le programme applique les heuristiques. */
        apply_heuristic (MOVE_PTR);
      }
    }
  }
  /* En dehors de la recherche de la variante principale, */
  else {

    for (i = 1; i <= moves_number; i++) {

      MOVE_PTR = &moves_list[i];

      /* le programme favorise les captures, */
      if (MOVE_PTR->captured) {

        /* en tenant compte de l'cart entre les valeurs des pices. */
        MOVE_PTR->note =   pieces_value[MOVE_PTR->captured & 0x7]
                         - pieces_value[board[MOVE_PTR->from] & 0x7] + 1000;

#ifdef DEBUG
debug.print (" (2.1 %d)", MOVE_PTR->note);
#endif /* DEBUG */
      }
      else {

        MOVE_PTR->note = 0;

#ifdef DEBUG
debug.print (" (2.2 %d)", MOVE_PTR->note);
#endif /* DEBUG */
      }

      /* Maintenant le programme applique les autres heuristiques. */
      apply_heuristic (MOVE_PTR);
    }
  }

#ifdef DEBUG
DB_SORTED_MOVES
#endif /* DEBUG */
}
/* End of function : sort_moves */

/*******************************************************************************
  Function     : update_killers
  Description  : Mise  jour des coups remarquables si un coup obtient un
                 meilleur score que celui d'un des coups prcdemment mmoriss.
  Parameters   : in  - coup  vrifier
                 in  - score de ce coup
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void update_killers (int moveid, int score) {

#ifdef DEBUG
debug.trace ("update_killers", true, &update_killers_calls);
#endif /* DEBUG */

  /* Le programme vrifie si le coup fait partie des trois meilleurs. */
  if (score > killers[search_ply].score1) {

    killers[search_ply].score2 = killers[search_ply].score1;
    killers[search_ply].score1 = score;

    killers[search_ply].moveid3 = killers[search_ply].moveid2;
    killers[search_ply].moveid2 = killers[search_ply].moveid1;
    killers[search_ply].moveid1 = moveid;

#ifdef DEBUG
DB_KILLER_MOVES
#endif /* DEBUG */
  }
  else if (score > killers[search_ply].score2) {

    killers[search_ply].score2 = score;

    killers[search_ply].moveid3 = killers[search_ply].moveid2;
    killers[search_ply].moveid2 = moveid;

#ifdef DEBUG
DB_KILLER_MOVES
#endif /* DEBUG */
  }
}
/* End of function : update_killers */
