/*******************************************************************************
  File : uci.c

  Description : Module de communication entre Sissa et toute interface graphique
                qui utilise le protocole UCI (Universal Chess Interface).
                Le codage de ce module respecte la norme UCI d'avril 2006
                Les fonctions et variables qui ne sont pas utilises dans cette
                version du programme sont indiques.
*******************************************************************************/

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte des diffrents modules du programme : */
#include "debug.h"
#include "hash.h"
#include "moves.h"
#include "main.h"
#include "tools.h"
#include "uci.h"

/*******************************************************************************
  Macro-instructions :
*******************************************************************************/

/* Pour factoriser l'analyse des paramtres de la commande go : */
#define parse_command() { \
  pointer = strtok (NULL, " "); \
  if (pointer == NULL) { \
    tools.send_output (true, "ERROR : Missing argument with uci command.\n"); \
    return; \
  } \
}

/*******************************************************************************
  Data :
*******************************************************************************/

bool uci_infinite;           /* not used */
bool uci_ponder;             /* not used */

int uci_binc;                /* additional time in msec for each move*/
int uci_btime;               /* black has x msec left on the clock */
int uci_movetime;            /* not used */
int uci_winc;                /* additional time in msec for each move */
int uci_wtime;               /* white has x msec left on the clock */

int uci_depth;               /* not used */
int uci_mate;                /* not used */
int uci_movestogo;           /* not used */
int uci_nodes;               /* not used */

/*******************************************************************************
  External functions :
*******************************************************************************/

/*******************************************************************************
  Function     : debug_command (not used)
  Description  : Switch the debug mode of the engine on and off.
  Parameters   : aucun
  Return value : aucune
  Validation   : .
*******************************************************************************/
void debug_command (void) {
}
/* End of function : debug_command */

/*******************************************************************************
  Function     : go
  Description  : Start calculating on the current position set up with the
                 "position" command.
  Parameters   : in  - informations sur le niveau de rflexion
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void go (char string[]) {

/* ------------------------------- Local data ------------------------------- */

const char *pointer;

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("go\n", true, &go_calls);
#endif /* DEBUG */

  /* To skip the first string (go) : */
  pointer = strtok (string," ");

  for (pointer = strtok (NULL, " ");
       pointer != NULL;
       pointer = strtok (NULL, " ")) {

    if (false) {

    } else if (!strcmp (pointer, "binc")) {

      parse_command();

      uci_binc = atoi (pointer);

    } else if (!strcmp (pointer, "btime")) {

      parse_command();

      uci_btime = atoi (pointer);

    } else if (!strcmp (pointer, "depth")) {

      parse_command();

      uci_depth = atoi (pointer);

    } else if (!strcmp (pointer, "infinite")) {

      uci_infinite = true;

    } else if (!strcmp (pointer, "mate")) {

      parse_command();

      uci_mate = atoi (pointer);

    } else if (!strcmp (pointer, "movestogo")) {

      parse_command();

      uci_movestogo = atoi (pointer);

    } else if (!strcmp (pointer, "movetime")) {

      parse_command();

      uci_movetime = atoi (pointer);

    } else if (!strcmp (pointer, "nodes")) {

      parse_command();

      uci_nodes = atoi (pointer);

    } else if (!strcmp (pointer, "ponder")) {

      uci_ponder = true;

    } else if (!strcmp (pointer, "searchmoves")) {

      /* Not implemented */

    } else if (!strcmp (pointer, "winc")) {

      parse_command();

      uci_winc = atoi (pointer);

    } else if (!strcmp (pointer, "wtime")) {

      parse_command();

      uci_wtime = atoi (pointer);
    }
  }

  /* Le programme joue pour la couleur au trait, */
  computer_color = active_color;

  /* et sa rflexion est lance. */
  main_module.start_thinking ();
}
/* End of function : go */

/*******************************************************************************
  Function     : info
  Description  : Display the evaluation and others informations from the engine,
                 according to the uci conventions in uci mode, otherwise in a
                 more human readable format.
  Parameters   : in  - valuation de la position
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void info (int score) {

/* ------------------------------- Local data ------------------------------- */

char lan_move[STRING_LENGTH]; /* Coup en notation algbrique. */

double elapsed_time_s;        /* Temps coul depuis le dbut de la rflexion
                                 (en secondes). */
double nps;                   /* Nombre de noeuds parcourus par secondes. */

int i;                        /* Indice de boucle. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("info\n", true, &info_calls);
#endif /* DEBUG */

  /* Calcul du temps coul depuis le dbut de la rflexion (en secondes). */
  elapsed_time_s = (clock () - start_thinking_time) / (double) CLOCKS_PER_SEC;

  if (uci_mode) {

    nps = (float) nodes / elapsed_time_s;

    /* En mode UCI sont affichs :
       - la profondeur de rflexion,
       - le score en centimes de points (conversion faite par l'interface),
       - le temps coul en ms (conversion faite par l'interface) ,
       - le nombre de noeuds parcourus,
       - le nombre de noeuds parcourus par secondes. */
    tools.send_output (true, "info depth %d score cp %d time %d nodes %d nps "
      "%d pv ", depth, score, (int) (elapsed_time_s * 1000), nodes, (int) nps);
  }
  else {

    /* En mode console sont affichs :
       - la profondeur de rflexion,
       - le score en centimes de points,
       - le temps coul en secondes,
       - le nombre de noeuds parcourus,
       - le nombre de noeuds parcourus en recherche de situation calme. */
    tools.send_output (true, "   %2d %5.2f %3d %8d %8d : ",
      depth, (float) score / 100, (int) elapsed_time_s, nodes, qnodes);
  }

  /* Affichage des sept premiers coups de la variante principale. */
  for (i = 1; i < pv_length[1] && i <= 7; i++) {

    /* Aprs conversion du coup en notation algbrique, */
    tools.convert_pgm_move (&pv[1][i], lan_move);

    /* il est envoy vers la sortie standard (mode console ou UCI). */
    tools.send_output (true, "%s ", lan_move);
  }

  tools.send_output (true, "\n");
}
/* End of function : info */

/*******************************************************************************
  Function     : isready
  Description  : This is used to synchronize the engine with the GUI.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void isready (void) {

#ifdef DEBUG
debug.trace ("isready\n", true, &isready_calls);
#endif /* DEBUG */

  /* Indique que le programme est en attente d'une commande en provenance
     de l'interface graphique : */
  tools.send_output (true, "readyok\n");
}
/* End of function : isready */

/*******************************************************************************
  Function     : ponderhit (not used)
  Description  : The user has played the expected move. This will be sent if the
                 engine was told to ponder on the same move the user has played.
  Parameters   : aucun
  Return value : aucune
  Validation   : .
*******************************************************************************/
void ponderhit (void) {
}
/* End of function : ponderhit */

/*******************************************************************************
  Function     : position
  Description  : Set up the position described in fenstring on the internal
                 board and play the moves on the internal chess board.
  Parameters   : in  - position de dpart
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void position (char string[]) {

/* ------------------------------- Local data ------------------------------- */

char lan_move[6];              /* Coup en notation algbrique. */
char *pointer;                 /* Pointeur pour parcourir la liste des coups. */
char position [STRING_LENGTH]; /* Position sur l'chiquier. */
char *position_moves;          /* Liste des coups jous depuis la position. */

int i;                         /* Indice de boucle. */

move_s move;                   /* Coup au format interne. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("position\n", true, &position_calls);
#endif /* DEBUG */

  /* Si la chane en entre contient le mot "startpos", */
  if (strstr (string, "startpos")) {

    /* alors la partie commence  partir de la position de dpart standard. */
    strcpy (position, start_position);
  }
  /* sinon la chane contient le mot "fen", */
  else if (strstr (string, "fen")) {

    /* et la partie commence  partir de la position donne aprs ce mot cl. */
    for (i = 13; i < strlen (string); i++) {

      position[i - 13] = string[i];
    }

    position[i - 13] = '\0';

    /* L'utilisation de la bibliothque des ouvertures n'est pas possible. */
    book_use = false;
  }

  /* Initialisation des donnes du programme  partir de la position entre. */
  main_module.initialize_game (position);

  /* Lecture des coups jous depuis la position ci-dessus. */
  if ((position_moves = strstr (string, "moves")) != NULL) {

    /* Initialisation de l'analyse de la chane et saut du mot "moves". */
    pointer = strtok (position_moves, " ");

    /* Tant que la fin de la liste n'est pas atteinte, */
    for (pointer = strtok (NULL, " ");
         pointer != NULL;
         pointer = strtok (NULL, " ")) {

      /* chaque coup trouv est converti, */
      strcpy (lan_move, pointer);
      tools.convert_lan_move (lan_move, &move);

      /* puis jou sur l'chiquier. */
      moves.make (&move);
    }

    /* Enfin la table des pices est mise  jour. */
    tools.update_pieces ();
  }
}
/* End of function : position */

/*******************************************************************************
  Function     : quit
  Description  : Quit the program as soon as possible.
  Parameters   : in  - code de retour
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void quit (int return_code) {

#ifdef DEBUG
debug.trace ("quit\n", true, &quit_calls);
#endif /* DEBUG */

#ifdef DEBUG
  /* Fermeture des fichiers utiliss pour dboguer le programme. */
  debug.close_debug_file ();
#endif /* DEBUG */

#ifdef DEBUG
  /* Ecriture d'informations, dans le fichier journal, sur le fonctionnement du
     programme. */
  debug.print_informations ();
  debug.print_statistics ();
#endif /* DEBUG */

  /* Si le fichier journal est utilis, */
  if (log_use) {

    /* alors il est ferm. */
    fclose (log_file);
  }

  /* En mode console, le programme attendra l'appui sur une touche pour se
     terminer. */
  if (!uci_mode) {

    printf ("\n");
    system ("pause");
  }

  /* Fin du programme. */
  exit (return_code);
}
/* End of function : quit */

/*******************************************************************************
  Function     : registration (not used)
  Description  : To try to register an engine.
  Parameters   : aucun
  Return value : aucune
  Validation   : .
*******************************************************************************/
void registration (void) {
}
/* End of function : registration */

/*******************************************************************************
  Function     : setoption (not used)
  Description  : This is sent to the engine when the user wants to change the
                 internal parameters of the engine.
  Parameters   : aucun
  Return value : aucune
  Validation   : .
*******************************************************************************/
void setoption (void) {
}
/* End of function : setoption */

/*******************************************************************************
  Function     : stop (not used)
  Description  : Stop calculating the engine as soon as possible.
  Parameters   : aucun
  Return value : aucune
  Validation   : .
*******************************************************************************/
void stop (void) {
}
/* End of function : stop */

/*******************************************************************************
  Function     : uci_command
  Description  : Tell the engine to use the universal chess interface.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void uci_command (void) {

#ifdef DEBUG
debug.trace ("uci_command\n", true, &uci_cmd_calls);
#endif /* DEBUG */

  /* Initialisation de quelques indicateurs en mode UCI : */
  forced_mode = true;
  show_board  = false;
  uci_mode    = true;

  /* Prsentation du programme en move UCI : */
  tools.send_output (true, "id name %s %s\n", PRODUCT_NAME, PRODUCT_VERSION);
  tools.send_output (true, "id author %s\n", PRODUCT_AUTHOR);
  tools.send_output (true, "uciok\n");
}
/* End of function : uci_command */

/*******************************************************************************
  Function     : ucinewgame
  Description  : This is sent to the engine when the next search (started with
                 "position" and "go") will be from a different game.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void ucinewgame (void) {

#ifdef DEBUG
debug.trace ("ucinewgame\n", true, &ucinewgame_calls);
#endif /* DEBUG */

  /* Initialisation d'une nouvelle partie  partir de la position standard : */
  main_module.initialize_game (start_position);

  /* Temps de base pour chaque coup : */
  base_time = 0;
}
/* End of function : ucinewgame */

/*******************************************************************************
  Object declaration :
*******************************************************************************/

uci_module_s uci = {

  debug_command,
  go,
  info,
  isready,
  ponderhit,
  position,
  quit,
  registration,
  setoption,
  stop,
  uci_command,
  ucinewgame
};
